@@ -19,10 +19,22 @@
 
 /*#define DEBUG*/
 
+#if defined(__GNUC__) || (defined(__SUNPRO_C) && __SUNPRO_C > 0x580)
+#define DBG_NONE(args...)
+ /**/
+#else							/* notdef __GNUC__ || __SUNPRO_C */
+extern void g_none(char *first, ...);
+#define DBG_NONE g_none
+#endif
+ /**/
+
+#define DBG_THREAD DBG_NONE
+
 #include "bluefish.h"
 #include "async_queue.h"
 
 
+
 /******************************** queue functions ********************************/
 
 
@@ -35,6 +47,8 @@ queue_init_full(Tasyncqueue * queue, guint max_worknum, gboolean lockmutex, gboo
 	queue->max_worknum = max_worknum;
 	queue->queuefunc = queuefunc;
 	queue->startinthread=startinthread;
+	queue->threads=NULL;
+	queue->lockmutex = lockmutex;
 	if (lockmutex)
 #if GLIB_CHECK_VERSION(2, 32, 0)
 		g_mutex_init(&queue->mutex);
@@ -67,25 +81,38 @@ queue_run(Tasyncqueue * queue)
 		queue->worknum++;
 		if (queue->startinthread) {
 			GThread *thread;
-			GError *gerror=NULL;
 			DEBUG_MSG("queue_run %p, create new thread, worknum now is %d\n",queue,queue->worknum);
-			thread = g_thread_create((GThreadFunc)queue->queuefunc, item, FALSE, &gerror);
-			queue->threads = g_slist_append(queue->threads, thread);
-		} else {
-			if (queue->lockmutex)
+			DBG_THREAD("about to start thread, have %d threads in queue->threads\n",g_slist_length(queue->threads));
+			/* the old and the new function:
+			GThread *g_thread_new (const gchar *name,             GThreadFunc func,              gpointer data);
+			GThread *g_thread_create (GThreadFunc func,                 gpointer data,                 gboolean joinable,                 GError **error);
+			*/
 #if GLIB_CHECK_VERSION(2, 32, 0)
+			thread = g_thread_new(NULL, (GThreadFunc)queue->queuefunc, item);
+#else
+			thread = g_thread_create((GThreadFunc)queue->queuefunc, item, TRUE, NULL);
+#endif
+			queue->threads = g_slist_append(queue->threads, thread);
+			DBG_THREAD("started thread %p, have %d threads in queue->threads\n",thread, g_slist_length(queue->threads));
+		} else {
+			if (queue->lockmutex) {
+#if GLIB_CHECK_VERSION(2, 32, 0)
+				DBG_THREAD("unlock queue %p\n",queue);
 				g_mutex_unlock(&queue->mutex);
 #else
 				g_static_mutex_unlock(&queue->mutex);
 #endif
+			}
 			DEBUG_MSG("queue_run %p, calling queuefunc(), worknum now is %d\n",queue,queue->worknum);
 			queue->queuefunc(item);
-			if (queue->lockmutex)
+			if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
 				g_mutex_lock(&queue->mutex);
+				DBG_THREAD("lock queue %p\n",queue);
 #else
 				g_static_mutex_lock(&queue->mutex);
 #endif
+			}
 		}
 		startednew=TRUE;
 	}
@@ -96,21 +123,25 @@ gboolean
 queue_worker_ready(Tasyncqueue * queue)
 {
 	gboolean startednew;
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
 		g_mutex_lock(&queue->mutex);
+		DBG_THREAD("lock queue %p\n",queue);
 #else
 		g_static_mutex_lock(&queue->mutex);
 #endif
+	}
 	queue->worknum--;
 	DEBUG_MSG("queue_worker_ready %p, len=%d, worknum=%d\n",queue,queue->q.length, queue->worknum);
 	startednew = queue_run(queue);
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
+		DBG_THREAD("unlock queue %p\n",queue);
 		g_mutex_unlock(&queue->mutex);
 #else
 		g_static_mutex_unlock(&queue->mutex);
 #endif
+	}
 	return startednew;
 }
 
@@ -121,15 +152,20 @@ queue_worker_ready_inthread(Tasyncqueue *queue)
 
 #if GLIB_CHECK_VERSION(2, 32, 0)
 	g_mutex_lock(&queue->mutex);
+	DBG_THREAD("lock queue %p\n",queue);
 #else
 	g_static_mutex_lock(&queue->mutex);
 #endif
 	
 	if (!queue->q.tail) {
+		gpointer thisthread = g_thread_self();
 		queue->worknum--;
-		DEBUG_MSG("queue_worker_ready_inthread %p, queue length %d, just return (end thread, worknum=%d)\n",queue,g_queue_get_length(&queue->q),queue->worknum);
-		queue->threads = g_slist_remove(queue->threads, g_thread_self());
+		DEBUG_MSG("queue_worker_ready_inthread queue=%p, queue length %d, just return (end thread, worknum=%d)\n",queue,g_queue_get_length(&queue->q),queue->worknum);
+		queue->threads = g_slist_remove(queue->threads, thisthread);
+		DBG_THREAD("removed thread self %p, have %d threads on queue->threads, will unref it\n",thisthread,g_slist_length(queue->threads));
 #if GLIB_CHECK_VERSION(2, 32, 0)
+		g_thread_unref(thisthread);
+		DBG_THREAD("unlock queue %p\n",queue);
 		g_mutex_unlock(&queue->mutex);
 #else
 		g_static_mutex_unlock(&queue->mutex);
@@ -137,8 +173,9 @@ queue_worker_ready_inthread(Tasyncqueue *queue)
 		return;
 	}
 	item = g_queue_pop_tail(&queue->q);
-	DEBUG_MSG("queue_worker_ready_inthread %p, queue length=%d, worknum=%d\n",queue,g_queue_get_length(&queue->q), queue->worknum);
+	DEBUG_MSG("queue_worker_ready_inthread queue=%p, queue length=%d, worknum=%d\n",queue,g_queue_get_length(&queue->q), queue->worknum);
 #if GLIB_CHECK_VERSION(2, 32, 0)
+	DBG_THREAD("unlock queue %p\n",queue);
 	g_mutex_unlock(&queue->mutex);
 #else
 	g_static_mutex_unlock(&queue->mutex);
@@ -149,21 +186,25 @@ queue_worker_ready_inthread(Tasyncqueue *queue)
 void
 queue_push(Tasyncqueue * queue, gpointer item)
 {
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
 		g_mutex_lock(&queue->mutex);
+		DBG_THREAD("lock queue %p\n",queue);
 #else
 		g_static_mutex_lock(&queue->mutex);
 #endif
+	}
 	g_queue_push_head(&queue->q, item);
 	DEBUG_MSG("queue_push %p, new queue length=%d, worknum=%d\n",queue,queue->q.length, queue->worknum);
 	queue_run(queue);
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
+		DBG_THREAD("unlock queue %p\n",queue);
 		g_mutex_unlock(&queue->mutex);
 #else
 		g_static_mutex_unlock(&queue->mutex);
 #endif
+	}
 }
 
 /* return TRUE if we found the item on the queue, FALSE if we did not find the item */
@@ -172,45 +213,57 @@ queue_remove(Tasyncqueue * queue, gpointer item)
 {
 	gboolean retval;
 	gint len;
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
 		g_mutex_lock(&queue->mutex);
+		DBG_THREAD("lock queue %p\n",queue);
 #else
 		g_static_mutex_lock(&queue->mutex);
 #endif
+	}
 	len = queue->q.length;
 	g_queue_remove(&queue->q, item);
 	retval = (len<queue->q.length);
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
+		DBG_THREAD("unlock queue %p\n",queue);
 		g_mutex_unlock(&queue->mutex);
 #else
 		g_static_mutex_unlock(&queue->mutex);
 #endif
+	}
 	return retval;
 }
 
 void
 queue_cancel(Tasyncqueue *queue, GFunc freefunc, gpointer user_data)
 {
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
 		g_mutex_lock(&queue->mutex);
+		DBG_THREAD("lock queue %p\n",queue);
 #else
 		g_static_mutex_lock(&queue->mutex);
 #endif
+	}
 	g_queue_foreach(&queue->q, freefunc, user_data);
 	g_queue_clear(&queue->q);
-	if (queue->lockmutex)
+	if (queue->lockmutex) {
 #if GLIB_CHECK_VERSION(2, 32, 0)
+		DBG_THREAD("unlock queue %p\n",queue);
 		g_mutex_unlock(&queue->mutex);
 #else
 		g_static_mutex_unlock(&queue->mutex);
 #endif
+	}
 	if (queue->startinthread) {
 		GSList *tmpslist;
+		DBG_THREAD("queue_cancel, have %d threads in queue->threads\n",g_slist_length(queue->threads));
 		for (tmpslist = queue->threads;tmpslist;tmpslist=g_slist_next(tmpslist)) {
-			g_thread_join(tmpslist->data);
+			DBG_THREAD("joining thread %p\n",tmpslist->data);
+			g_thread_join(tmpslist->data); /* will unref the thread autyomatically */
 		}
+		g_slist_free(queue->threads);
+		queue->threads = NULL;
 	}
 }
