@@ -1509,21 +1509,6 @@ callback_remove_all(GSList **slist) {
 	*slist=NULL;
 }
 
-#ifndef HAVE_STRCASESTR
-#include <ctype.h>
-char *strcasestr(char *a, char *b)
-{
-	size_t l;
-	char f[3];
-
-	snprintf(f, sizeof(f), "%c%c", tolower(*b), toupper(*b));
-	for (l = strcspn(a, f); l != strlen(a); l += strcspn(a + l + 1, f) + 1)
-		if (strncasecmp(a + l, b, strlen(b)) == 0)
-			return(a + l);
-	return(NULL);
-}
-#endif
-
 Telist *
 bf_elist_prepend(gpointer cur, gpointer new)
 {
@@ -1595,3 +1580,32 @@ bf_elist_remove(gpointer toremove)
 	BF_ELIST(toremove)->prev = NULL;
 	return ret;
 }
+
+void print_character_escaped(gunichar uc)
+{
+	if (uc > 127) {
+		gchar output[10];
+		output[0] = '\0';
+		g_unichar_to_utf8(uc,output);
+		g_print("%s ",output);
+	}
+	switch (uc) {
+	case '\0':
+		g_print("\\0");
+		break;
+	case '\n':
+		g_print("\\n");
+		break;
+	case '\r':
+		g_print("\\r");
+		break;
+	case '\t':
+		g_print("\\t");
+		break;
+	case '\a':
+		g_print("\\a");
+		break;
+	default:
+		g_print("%c ",uc);
+	}
+}
