@@ -42,8 +42,9 @@
 #define DBG_MSG g_print*/
 
 typedef struct {
-	GHashTable *patterns;
-	GHashTable *contexts;
+	GHashTable *patterns; /* key is the id, value is the number */
+	GHashTable *contexts; /* key is the id, value is the number */
+	Tlangdebug ldb;
 	Tscantable *st;				/* while not finished */
 	GHashTable *commentid_table;	/* a hash table with a comment ID string as key and an integer position in the GArray as value */
 	gchar *smartindentchars;
@@ -131,6 +132,43 @@ arr2_hash(gconstpointer v)
 
 /* langmgr code */
 
+gchar *
+ldb_stack_string(gpointer ldb)
+{
+	GList *tmplist = g_list_last(((Tlangdebug *)ldb)->stack);
+	GString *str;
+	/*g_print("ldb_stack_string, called for ldb %p, have %d entries on stack\n",ldb, g_list_length(((Tlangdebug *)ldb)->stack));*/
+	if (!tmplist)
+		return g_strdup("");
+	str = g_string_new(tmplist->data);
+	while(tmplist) {
+		g_string_append(str," / ");
+		g_string_append(str,tmplist->data);
+		tmplist = g_list_previous(tmplist);
+	}
+	return g_string_free(str, FALSE);
+}
+
+static void
+ldb_stack_push(Tlangdebug *ldb, const gchar *string)
+{
+	ldb->stack = g_list_prepend(ldb->stack, g_strdup(string));
+	/*g_print("langdebug stack push: have %d entries on stack, pushed %s\n",g_list_length(ldb->stack),string);*/
+}
+
+static void
+ldb_stack_pop(Tlangdebug *ldb)
+{
+	GList *tmplist = ldb->stack;
+	if (!tmplist) {
+		g_warning("ldb_stack_pop, noting on the stack\n");
+		return;
+	}
+	/*g_print("langdebug stack pop: have %d entries on stack, popped %s\n",g_list_length(ldb->stack)-1,(gchar *)tmplist->data);*/
+	g_free(tmplist->data);
+	ldb->stack = g_list_delete_link(ldb->stack, tmplist);
+}
+
 static void
 skip_to_end_tag(xmlTextReaderPtr reader, int depth)
 {
@@ -445,7 +483,7 @@ build_lang_finished_lcb(gpointer data)
 	g_hash_table_destroy(bfparser->patterns);
 	g_hash_table_destroy(bfparser->contexts);
 	g_hash_table_destroy(bfparser->commentid_table);
-	/*g_hash_table_destroy(bfparser->setoptions); */
+	g_list_free_full(bfparser->ldb.stack, g_free);
 	g_slice_free(Tbflangparsing, bfparser);
 	return FALSE;
 }
@@ -590,7 +628,7 @@ process_header(xmlTextReaderPtr reader, Tbflang * bflang)
 				} else if (style) {	/* no textstyle was configured, set the provided style */
 					if (!gtk_text_tag_table_lookup(langmgr.tagtable, style)) {
 						gchar *arr[] = { style, "", "", "", "", NULL };
-						g_print("Possible error in language file, textstyle %s does not exist\n", style);
+						g_print("Possible error in language file header: textstyle %s does not exist\n", style);
 						main_v->props.textstyles = g_list_prepend(main_v->props.textstyles, g_strdupv(arr));
 					}
 					g_hash_table_insert(langmgr.configured_styles,
@@ -675,10 +713,11 @@ add_autocomplete(Tbflangparsing * bfparser, guint16 matchnum, GSList *list, cons
 			gboolean freestring=FALSE;
 			gint backup_cursor = ac->backup_cursor;
 			if (ac->string == NULL && tag && ac->mode == ac_mode_string) {
-				string = g_strconcat("></", tag, ">", NULL);
-				backup_cursor = strlen(string)-1;
+				string = g_strconcat("<",tag,">", NULL);
+				backup_cursor = 0;
 				freestring=TRUE;
 			}
+			/*g_print("line %d: autocomplete %s %s for %d\n",__LINE__,(ac->mode == ac_mode_string) ? "set" : "append", string, matchnum);*/
 			match_add_autocomp_item(bfparser->st, matchnum,
 										(ac->mode == ac_mode_string) ? string : NULL,
 										(ac->mode == ac_mode_append) ? string : NULL,
@@ -786,6 +825,8 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 					{"condition_relation", &condition_relation, attribtype_int},
 					};
 	parse_attributes(bfparser->bflang,reader, attribs, 23);
+	ldb_stack_push(&bfparser->ldb, id?id:(idref?idref:(pattern?pattern:"empty-element")));
+
 #ifdef DEVELOPMENT
 /*	if (g_strcmp0(id,"e.css.in_style_attribute.dquote")==0) {
 		g_print("e.css.in_style_attribute.dquote has condition_mode=%d and contextref=%s\n",condition_mode,condition_contextref);
@@ -808,14 +849,16 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 			if (!matchnum && !pattern) {
 				g_print("Error in language file, element with id %s does not exist\n", idref);
 			} else if (matchnum) {
-				compile_existing_match(bfparser->st, matchnum, context);
+				compile_existing_match(bfparser->st, matchnum, context, &bfparser->ldb);
 				used_idref=TRUE;
 				if (g_array_index(bfparser->st->matches, Tpattern, matchnum).nextcontext < 0
 					&& (-1 * g_array_index(bfparser->st->matches, Tpattern, matchnum).nextcontext) >=
-					g_queue_get_length(contextstack)) {
+					g_queue_get_length(contextstack) && (condition_mode== 0)) {
+					gchar *dbstring = ldb_stack_string(&bfparser->ldb);
 					g_print
-						("Possible error in language file, idref %s ends_context=%d, but has only %d parent contexts\n",
+						("Possible error in language file %s: idref %s ends_context=%d, but has only %d parent contexts\n",dbstring,
 						 idref, ends_context, g_queue_get_length(contextstack));
+					g_free(dbstring);
 				}
 
 			}
@@ -828,10 +871,12 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 			if (ends_context) {
 				/* the nth number in the stack */
 				nextcontext = -1 * ends_context;	/*GPOINTER_TO_INT(g_queue_peek_nth(contextstack,ends_context)); */
-				if (ends_context >= g_queue_get_length(contextstack)) {
+				if (ends_context >= g_queue_get_length(contextstack) && (condition_mode== 0)) {
+					gchar *dbstring = ldb_stack_string(&bfparser->ldb);
 					g_print
-						("Possible error in language file, id %s / pattern %s has ends_context=%d, but has only %d parent contexts\n",
-						 id?id:"-", pattern?pattern:"null", ends_context, g_queue_get_length(contextstack));
+						("Possible error in language file %s: id %s / pattern %s has ends_context=%d, but has only %d parent contexts\n",
+						 dbstring,id?id:"-", pattern?pattern:"null", ends_context, g_queue_get_length(contextstack));
+					g_free(dbstring);
 				}
 			}
 			if (blockstartelement) {
@@ -848,7 +893,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 								pattern,
 								is_regex != UNDEFINED ? is_regex : (ih_is_regex !=UNDEFINED ? ih_is_regex : FALSE),
 								case_insens != UNDEFINED ? case_insens : (ih_case_insens != UNDEFINED ? ih_case_insens :FALSE),
-								context);
+								context, &bfparser->ldb);
 			if (block_name) {
 				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
 			}
@@ -871,7 +916,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 				gchar *refname=NULL;
 				if (condition_mode ==1 || condition_mode == 2) {
 					refname = condition_contextref;
-				} else if (condition_mode ==1 || condition_mode == 2) {
+				} else if (condition_mode ==3 || condition_mode == 4) {
 					refname = condition_idref;
 				}
 				DBG_PARSING("condition_mode=%d, call pattern_set_condition, refname %s, contextref=%s\n",condition_mode,refname,condition_contextref);
@@ -894,7 +939,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 					name = xmlTextReaderName(reader);
 					if (xmlStrEqual(name, (xmlChar *) "context")) {
 						if (processed_context) {
-							g_print("Error in language file: element %s / %s has multiple inner contexts\n",id,pattern);
+							g_print("Error in language file: element %s with pattern %s has multiple inner contexts\n",id?id:"without id",pattern);
 						}
 						DBG_PARSING("in pattern, found countext\n");
 						nextcontext = process_scanning_context(reader, bfparser, contextstack);
@@ -931,6 +976,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 	}
 	/* TODO cleanup! */
 	free_autocomplete(autocomplete);
+	ldb_stack_pop(&bfparser->ldb);
 
 	if (pattern)
 		xmlFree(pattern);
@@ -944,6 +990,7 @@ process_scanning_element(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gin
 	/*if (blockhighlight) xmlFree(blockhighlight); */
 	if (class)
 		xmlFree(class);
+
 	return matchnum;
 }
 
@@ -963,11 +1010,11 @@ add_string(Tbflangparsing * bfparser, guint16 contexttag, const gchar *stringnam
 	gchar *contextname = g_strconcat(stringname, ".context", NULL);
 	strcontext = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->contexts, contextname));
 	if (strcontext == 0) {
-		strcontext = new_context(bfparser->st, 4, bfparser->bflang->name, (gchar *)singlematch, stringhighlight, FALSE, FALSE);
-		endmatch = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, strcontext);
+		strcontext = new_context(bfparser->st, 4, bfparser->bflang->name, (gchar *)singlematch, stringhighlight, FALSE, FALSE, FALSE);
+		endmatch = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, strcontext, &bfparser->ldb);
 		pattern_set_runtime_properties(bfparser->st, endmatch, stringhighlight,-1,FALSE,FALSE,0,FALSE,FALSE);
 	}
-	matchstring = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, contexttag);
+	matchstring = add_pattern_to_scanning_table(bfparser->st, singlematch, FALSE, FALSE, contexttag, &bfparser->ldb);
 	pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,strcontext,FALSE,FALSE,0,FALSE,FALSE);
 	if (val && val[0]=='1') {
 		pattern_set_blockmatch(bfparser->st, matchstring,TRUE,FALSE,0,NULL,NULL,FALSE);
@@ -986,7 +1033,7 @@ add_attribute_to_tag(Tbflangparsing * bfparser, const gchar *attrstring, gint co
 {
 	guint16 attrmatch;
 	if (strchr(attrstring, '=')== NULL) {
-		attrmatch = add_pattern_to_scanning_table(bfparser->st, attrstring, FALSE, TRUE, contexttag);
+		attrmatch = add_pattern_to_scanning_table(bfparser->st, attrstring, FALSE, TRUE, contexttag, &bfparser->ldb);
 		pattern_set_runtime_properties(bfparser->st, attrmatch,attribhighlight, 0, FALSE, FALSE,0, FALSE, FALSE);
 		match_add_autocomp_item(bfparser->st, attrmatch, NULL,attrib_autocomplete_append,attrib_autocomplete_backup_cursor);
 		match_autocomplete_reference(bfparser->st, attrmatch, contexttag);
@@ -1002,7 +1049,7 @@ add_attribute_to_tag(Tbflangparsing * bfparser, const gchar *attrstring, gint co
 			g_warning("attribute %s refers to context %s which does not (yet) exist\n",splitted[0],splitted[1]);
 		}
 		tmp = g_strdup_printf("%s[ \t]*=", splitted[0]);
-		attrmatch = add_pattern_to_scanning_table(bfparser->st, tmp, TRUE, TRUE, contexttag);
+		attrmatch = add_pattern_to_scanning_table(bfparser->st, tmp, TRUE, TRUE, contexttag, &bfparser->ldb);
 		g_free(tmp);
 		pattern_set_runtime_properties(bfparser->st, attrmatch,attribhighlight,startscontext,FALSE,FALSE,0,FALSE,FALSE);
 		match_add_autocomp_item(bfparser->st, attrmatch, splitted[0],attrib_autocomplete_append,attrib_autocomplete_backup_cursor);
@@ -1042,6 +1089,9 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
 	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 14 : 12);
+
+	ldb_stack_push(&bfparser->ldb, id?id:"nameless-tag");
+
 	add_tag = do_parse(bfparser, class, notclass);
 	if (add_tag) {
 		if (idref && idref[0] && !tag) {
@@ -1051,7 +1101,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			}
 			DBG_PARSING("lookup tag with id %s returned matchnum %d\n", id, matchnum);
 			if (matchnum)
-				compile_existing_match(bfparser->st, matchnum, context);
+				compile_existing_match(bfparser->st, matchnum, context, &bfparser->ldb);
 		} else if (tag && tag[0]) {
 			gchar *attrib_context_id = NULL;
 			gchar **attrib_arr = NULL;
@@ -1088,7 +1138,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				}
 			}
 			tmp = g_strconcat("<", tag, NULL);
-			matchnum = add_pattern_to_scanning_table(bfparser->st, tmp, FALSE, case_insens, context);
+			matchnum = add_pattern_to_scanning_table(bfparser->st, tmp, FALSE, case_insens, context, &bfparser->ldb);
 			if (block_name) {
 				foldable = lookup_block_foldable(bfparser->bflang->name, block_name);
 			}
@@ -1098,7 +1148,9 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 								contexttag, TRUE, FALSE,0, FALSE, FALSE);
 			DBG_PARSING("insert tag %s into hash table with matchnum %d\n", id ? id : tmp, matchnum);
 			if (g_hash_table_lookup(bfparser->patterns, id ? id : tmp) != NULL) {
-				g_warning("Possible error in language file, pattern with id %s already exists\n", id ? id : tmp);
+				gchar *dbstring = ldb_stack_string(&bfparser->ldb);
+				g_warning("Possible error in language file %s: pattern with id %s already exists\n", dbstring, id ? id : tmp);
+				g_free(dbstring);
 			} else {
 				g_hash_table_insert(bfparser->patterns, g_strdup(id ? id : tmp),
 									GINT_TO_POINTER((gint) matchnum));
@@ -1108,7 +1160,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			if (!contexttag) {
 				static const gchar *internal_tag_string_d = "__internal__.e.tag.attribute.string.d";
 				static const gchar *internal_tag_string_s = "__internal__.e.tag.attribute.string.s";
-				contexttag = new_context(bfparser->st, 8, bfparser->bflang->name, ">\"=' \t\n\r<", NULL, FALSE, FALSE);
+				contexttag = new_context(bfparser->st, 8, bfparser->bflang->name, ">\"=' \t\n\r<", NULL, FALSE, FALSE, FALSE);
 				match_set_nextcontext(bfparser->st, matchnum, contexttag);
 				if (attrib_arr) {
 					gchar **tmp2;
@@ -1125,54 +1177,33 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				matchstring = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->patterns, internal_tag_string_d));
 				if (matchstring) {
 					/*g_print("adding existing tag string %s to context %d\n",internal_tag_string_d,contexttag); */
-					compile_existing_match(bfparser->st, matchstring, contexttag);
+					compile_existing_match(bfparser->st, matchstring, contexttag, &bfparser->ldb);
 				} else {
 					add_string(bfparser, contexttag, internal_tag_string_d, "\"", "\"[^\"]*\"");
-/*					gchar *val = lookup_user_option(bfparser->bflang->name, "attribute_string_is_block");
-					if (val && val[0]=='1') {
-						guint16 strcontext, endmatch;
-						strcontext = new_context(bfparser->st, 4, bfparser->bflang->name, "\"", stringhighlight, FALSE);
-						matchstring = add_pattern_to_scanning_table(bfparser->st, "\"", FALSE, FALSE, contexttag);
-						pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,strcontext,FALSE,FALSE,0,FALSE,FALSE);
-						pattern_set_blockmatch(bfparser->st, matchstring,TRUE,FALSE,0,NULL,NULL,FALSE);
-						endmatch = add_pattern_to_scanning_table(bfparser->st, "\"", FALSE, FALSE, strcontext);
-						pattern_set_runtime_properties(bfparser->st, endmatch, stringhighlight,-1,FALSE,FALSE,0,FALSE,FALSE);
-						pattern_set_blockmatch(bfparser->st, endmatch,FALSE,TRUE,matchstring,NULL,NULL,FALSE);
-					} else {
-						matchstring = add_pattern_to_scanning_table(bfparser->st, "\"[^\"]*\"", TRUE, FALSE, contexttag);
-						pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,
-									0, FALSE, FALSE,0, FALSE, FALSE);
-					}
-					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_d),
-										GINT_TO_POINTER((gint) matchstring));*/
 				}
 				matchstring = GPOINTER_TO_INT(g_hash_table_lookup(bfparser->patterns, internal_tag_string_s));
 				if (matchstring) {
 					/*g_print("adding existing tag string %s to context %d\n",internal_tag_string_s,contexttag); */
-					compile_existing_match(bfparser->st, matchstring, contexttag);
+					compile_existing_match(bfparser->st, matchstring, contexttag, &bfparser->ldb);
 				} else {
 					add_string(bfparser, contexttag, internal_tag_string_s, "'", "'[^']*'");
-/*					static const gchar *stringhighlight = "attribute-string";
-					matchstring = add_pattern_to_scanning_table(bfparser->st, "'[^']*'", TRUE, FALSE, contexttag);
-					pattern_set_runtime_properties(bfparser->st, matchstring, stringhighlight,
-								0, FALSE, FALSE,0, FALSE, FALSE);
-					g_hash_table_insert(bfparser->patterns, g_strdup(internal_tag_string_s),
-										GINT_TO_POINTER((gint) matchstring));*/
 				}
 
 				if (!sgml_shorttag) {
-					guint16 tmpnum = add_pattern_to_scanning_table(bfparser->st, "/>", FALSE, FALSE, contexttag);
+					guint16 tmpnum = add_pattern_to_scanning_table(bfparser->st, "/>", FALSE, FALSE, contexttag, &bfparser->ldb);
 					pattern_set_runtime_properties(bfparser->st, tmpnum,
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, FALSE,0, FALSE, FALSE);
 					pattern_set_blockmatch(bfparser->st, tmpnum,
 								FALSE,TRUE, -1,NULL,NULL,TRUE);
-					if (bfparser->autoclose_tags)
+					if (bfparser->autoclose_tags) {
 						match_add_autocomp_item(bfparser->st, tmpnum, NULL, NULL, 0);
+						/*g_print("line %d: adding default autocomp string %s to %d\n",__LINE__,"/>",tmpnum);*/
+					}
 					match_autocomplete_reference(bfparser->st, tmpnum, contexttag);
 				}
 				tmp = no_close ? NULL : g_strconcat("</", tag, ">", NULL);
-				starttagmatch = add_pattern_to_scanning_table(bfparser->st, ">", FALSE, FALSE, contexttag);
+				starttagmatch = add_pattern_to_scanning_table(bfparser->st, ">", FALSE, FALSE, contexttag, &bfparser->ldb);
 				pattern_set_runtime_properties(bfparser->st, starttagmatch,
 								highlight ? highlight : ih_highlight,
 								-1, FALSE, bfparser->stretch_tag_block,0, FALSE, FALSE);
@@ -1180,8 +1211,12 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				if (bfparser->autoclose_tags && !no_close) {
 					gchar *tmp2;
 					tmp2 = g_strconcat("></", tag, ">", NULL);
+					/* add the closing tag to the inside context of the tag */
 					match_add_autocomp_item(bfparser->st, starttagmatch, NULL, tmp, tmp ? strlen(tmp) : 0);
+					/*g_print("line %d: adding %s autocomp append to %d\n",__LINE__,tmp,starttagmatch);*/
+					/* add autocomplete to the tag pattern itself with closing tag */
 					match_add_autocomp_item(bfparser->st, matchnum, NULL, tmp2, strlen(tag)+3);
+					/*g_print("line %d: adding %s autocomp append to %d\n",__LINE__,tmp2,matchnum);*/
 				}
 				if (tmp)
 					g_free(tmp);
@@ -1233,10 +1268,12 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				}
 			}
 			if (!no_close && autocomplete == NULL && ih_autocomplete == NULL) {
-				gchar *tmp2 = g_strconcat("></", tag, ">", NULL);
-				match_add_autocomp_item(bfparser->st, matchnum, NULL, tmp2, strlen(tag) + 3);
-				g_free(tmp2);
+				/*gchar *tmp2 = g_strconcat("></", tag, ">", NULL);*/
+				match_add_autocomp_item(bfparser->st, matchnum, NULL, ">", strlen(tag) + 3);
+				/*g_print("line %d: adding %s autocomp append to %d\n",__LINE__,">",matchnum);*/
+				/*g_free(tmp2);*/
 			} else {
+				/*g_print("line %d: call add_autocomplete for %d\n",__LINE__,matchnum);*/
 				add_autocomplete(bfparser, matchnum, autocomplete?autocomplete:ih_autocomplete, tag);
 				/*match_add_autocomp_item(bfparser->st, matchnum, NULL,
 										autocomplete_append ? autocomplete_append : ih_autocomplete_append,
@@ -1252,7 +1289,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 			match_autocomplete_reference(bfparser->st, matchnum, context);
 			if (!no_close) {
 				tmp = g_strconcat("</", tag, ">", NULL);
-				endtagmatch = add_pattern_to_scanning_table(bfparser->st, tmp, FALSE, case_insens , innercontext);
+				endtagmatch = add_pattern_to_scanning_table(bfparser->st, tmp, FALSE, case_insens , innercontext, &bfparser->ldb);
 				pattern_set_runtime_properties(bfparser->st, endtagmatch,
 									highlight ? highlight : ih_highlight,
 									(innercontext == context) ? 0 : -2,
@@ -1263,7 +1300,9 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 				match_add_autocomp_item(bfparser->st, endtagmatch, NULL, NULL, 0);
 				match_autocomplete_reference(bfparser->st, endtagmatch, innercontext);
 				if (g_hash_table_lookup(bfparser->patterns, tmp)) {
-					g_print("Possible error in language file: id %s already exists\n", tmp);
+					gchar *dbstring = ldb_stack_string(&bfparser->ldb);
+					g_print("Possible error in language file %s: id %s already exists\n", dbstring, tmp);
+					g_free(dbstring);
 				} else {
 					g_hash_table_insert(bfparser->patterns, g_strdup(tmp),
 										GINT_TO_POINTER((gint) endtagmatch));
@@ -1282,7 +1321,7 @@ process_scanning_tag(xmlTextReaderPtr reader, Tbflangparsing * bfparser, guint16
 	g_free(attrib_autocomplete_append);
 	/*g_free(attribhighlight); */
 	g_free(attributes);
-
+	ldb_stack_pop(&bfparser->ldb);
 	return matchnum;
 }
 
@@ -1314,6 +1353,9 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 					{"attrib_autocomplete_append", &attrib_autocomplete_append, attribtype_string},
 					{"attrib_autocomplete_backup_cursor", &attrib_autocomplete_backup_cursor, attribtype_int}};
 	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 8 : 6);
+
+	ldb_stack_push(&bfparser->ldb, "group");
+
 	add_group = do_parse(bfparser, class, notclass);
 	DBG_PARSING("add_group=%d for class=%s and notclass=%s\n",add_group, class, notclass);
 	if (!add_group) {
@@ -1377,6 +1419,7 @@ process_scanning_group(xmlTextReaderPtr reader, Tbflangparsing * bfparser, gint
 	g_free(notclass);
 	g_free(attrib_autocomplete_append);
 	/*g_free(attribhighlight); */
+	ldb_stack_pop(&bfparser->ldb);
 }
 
 static guint8
@@ -1419,8 +1462,8 @@ static gint16
 process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQueue * contextstack)
 {
 	gchar *symbols = NULL, *highlight = NULL, *id = NULL, *idref = NULL, *commentid_block =
-		NULL, *commentid_line = NULL;
-	gboolean autocomplete_case_insens = FALSE, isempty;
+		NULL, *commentid_line = NULL, *dump_dfa_chars=NULL;
+	gboolean autocomplete_case_insens = FALSE, isempty, dump_dfa_run=FALSE;
 	gint default_spellcheck = SPELLCHECK_INHERIT;
 	gint context, depth;
 
@@ -1431,7 +1474,10 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 					{"commentid_block", &commentid_block, attribtype_string},
 					{"commentid_line", &commentid_line, attribtype_string},
 					{"default_spellcheck", &default_spellcheck, attribtype_boolean},
-					{"autocomplete_case_insens", &autocomplete_case_insens, attribtype_boolean}};
+					{"autocomplete_case_insens", &autocomplete_case_insens, attribtype_boolean},
+					{"dump_dfa_chars", &dump_dfa_chars, attribtype_string},
+					{"dump_dfa_run", &dump_dfa_run, attribtype_boolean},
+					};
 #ifdef HAVE_LIBENCHANT
 	if (g_queue_get_length(contextstack)==0) {
 		DBG_PARSING("top level context, set default_spellcheck to %d\n", bfparser->default_spellcheck);
@@ -1440,8 +1486,10 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 #endif
 	depth = xmlTextReaderDepth(reader);
 	isempty = xmlTextReaderIsEmptyElement(reader);
+	parse_attributes(bfparser->bflang,reader, attribs, 10);
+
+	ldb_stack_push(&bfparser->ldb, id?id:"nameless-context");
 
-	parse_attributes(bfparser->bflang,reader, attribs, bfparser->load_completion ? 7 : 6);
 	DBG_PARSING("found <context> with id=%s, idref=%s\n", id, idref);
 	if (idref && idref[0]) {
 		DBG_PARSING("lookup context %s in hash table..\n", idref);
@@ -1450,8 +1498,8 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 			g_print("Error in language file: context with id %s does not exist\n",idref);
 		}
 #ifdef DEVELOPMENT
-		if (context == 0) {
-			g_print("Context has idref=%s which does not exist, isempty=%d, id=%s\n",idref,isempty,id);
+		if (context == 0 && (!id || isempty)) {
+			g_print("Context has idref=%s, which does not exist, isempty=%d, id=%s\n",idref,isempty,id);
 		}
 #endif
 		g_free(idref);
@@ -1464,16 +1512,19 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 			g_free(highlight);
 			g_free(commentid_block);
 			g_free(commentid_line);
+			g_free(dump_dfa_chars);
+			ldb_stack_pop(&bfparser->ldb);
 			return context;
 		}
 	}
 	if (!symbols) {
 		g_warning("language file has context without symbols, abort.");
+		ldb_stack_pop(&bfparser->ldb);
 		return 0;
 	}
 	/* create context */
 	DBG_PARSING("create context %s, symbols %s and highlight %s\n", id, symbols, highlight);
-	context = new_context(bfparser->st, 32, bfparser->bflang->name, symbols, highlight, autocomplete_case_insens, default_spellcheck);
+	context = new_context(bfparser->st, 32, bfparser->bflang->name, symbols, highlight, autocomplete_case_insens, default_spellcheck, dump_dfa_run);
 	g_queue_push_head(contextstack, GINT_TO_POINTER(context));
 	if (id) {
 		DBG_PARSING("insert context %s into hash table as %d\n", id, context);
@@ -1507,9 +1558,14 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 								   UNDEFINED, UNDEFINED, 0, NULL);
 		} else if (xmlStrEqual(name, (xmlChar *) "context")) {
 			if (depth == xmlTextReaderDepth(reader)) {
+				if (dump_dfa_chars) {
+					print_DFA_subset(bfparser->st, context, dump_dfa_chars);
+					g_free(dump_dfa_chars);
+				}
 				xmlFree(name);
 				DBG_PARSING("parsing context, end-of-context, return context %d\n", context);
 				g_queue_pop_head(contextstack);
+				ldb_stack_pop(&bfparser->ldb);
 				return context;
 			} else {
 				/* a context inside a context is usually a context that is referred to by various other elements or tags */
@@ -1523,6 +1579,7 @@ process_scanning_context(xmlTextReaderPtr reader, Tbflangparsing * bfparser, GQu
 	/* can we ever get here ?? */
 	DBG_PARSING("pop context %d\n", GPOINTER_TO_INT(g_queue_peek_head(contextstack)));
 	g_queue_pop_head(contextstack);
+	ldb_stack_pop(&bfparser->ldb);
 	return context;
 }
 
@@ -1530,20 +1587,33 @@ static void
 bftextview2_match_conditions(Tbflangparsing * bfparser) {
 	gint i;
 	GHashTable *ht;
-	g_print("bftextview2_match_conditions, have %d conditions\n",bfparser->st->conditions->len);
+	const gchar *relationtypestring;
+	DBG_MSG("bftextview2_match_conditions, have %d conditions\n",bfparser->st->conditions->len);
 	for (i = 1; i < bfparser->st->conditions->len; i++) {
-		g_print("bftextview2_match_conditions,running condition %d with refname %s\n",i,g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname);
+		DBG_PARSING("bftextview2_match_conditions,running condition %d with refname %s\n",i,g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname);
 		if (!g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname) {
 			g_warning("Error in language file, condition refers to non existing id %s",g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname);
 			continue;
 		}
 		if (g_array_index(bfparser->st->conditions, Tpattern_condition, i).relationtype == 1 || g_array_index(bfparser->st->conditions, Tpattern_condition, i).relationtype == 2) {
 			 ht = bfparser->contexts;
+			 relationtypestring = "context";
 		} else {
 			 ht = bfparser->patterns;
+			 relationtypestring = "element";
 		}
 		g_array_index(bfparser->st->conditions, Tpattern_condition, i).ref = GPOINTER_TO_INT(g_hash_table_lookup(ht, g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname));
-		g_print("bftextview2_match_conditions, set ref %d for %s\n",g_array_index(bfparser->st->conditions, Tpattern_condition, i).ref, g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname);
+		if (g_array_index(bfparser->st->conditions, Tpattern_condition, i).ref==0) {
+			gint j;
+			/* find out which pattern has this condition to easy debugging of the language file */
+			for (j = 1; j < bfparser->st->matches->len; j++) {
+				if (g_array_index(bfparser->st->matches, Tpattern, j).condition == i) {
+					g_print("Error in language file %s: condition reference to %s %s for pattern %d (%s) does not exist\n",bfparser->bflang->name,relationtypestring,g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname, 
+						j, g_array_index(bfparser->st->matches, Tpattern, j).pattern);
+				}
+			}
+		}
+		DBG_MSG("bftextview2_match_conditions, set ref %d for %s\n",g_array_index(bfparser->st->conditions, Tpattern_condition, i).ref, g_array_index(bfparser->st->conditions, Tpattern_condition, i).refname);
 	}
 }
 
@@ -1557,13 +1627,10 @@ build_lang_thread(gpointer data)
 	GList *tmplist;
 
 	bfparser = g_slice_new0(Tbflangparsing);
+	bfparser->ldb.stack = NULL;
 	bfparser->patterns = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
 	bfparser->contexts = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
-	/*bfparser->setoptions =  g_hash_table_new_full(g_str_hash,g_str_equal,g_free,NULL); */
 	bfparser->bflang = bflang;
-	/*for(tmplist = g_list_first(bfparser->bflang->setoptions);tmplist;tmplist=g_list_next(tmplist)) {
-	   g_hash_table_insert(bfparser->setoptions,g_strdup(tmplist->data),GINT_TO_POINTER(1));
-	   } */
 	bfparser->st = scantable_new(bflang->size_table, bflang->size_matches, bflang->size_contexts);
 	bfparser->commentid_table = g_hash_table_new_full(g_str_hash, g_str_equal, xmlFree, NULL);
 
@@ -1678,7 +1745,10 @@ build_lang_thread(gpointer data)
 		xmlFree(name);
 	}
 	xmlFreeTextReader(reader);
-	/* do some final memory management */
+	/************************
+	 * finished parsing the XML,
+	 * do some final memory management
+	 ************************/
 	if (bfparser->st) {
 		gint i, tablenum=0, largest_table=0;
 
@@ -1816,7 +1886,7 @@ parse_bflang2_header(const gchar * filename)
 	return bflang;
 }
 
-GList *
+/*GList *
 langmgr_get_languages_mimetypes(void)
 {
 	GList *retlist = NULL, *tmplist = g_list_first(langmgr.bflang_list);
@@ -1826,6 +1896,46 @@ langmgr_get_languages_mimetypes(void)
 		tmplist = tmplist->next;
 	}
 	return retlist;
+}*/
+
+GList *
+langmgr_get_languages_mimetypes(void)
+{
+	GList *retlist = NULL, *tmplist = g_list_first(langmgr.bflang_list);
+
+	while (tmplist) {
+		if (((Tbflang *) (tmplist->data))->mimetypes) {
+			GList *llist = g_list_first(((Tbflang *) (tmplist->data))->mimetypes);
+			while (llist) {
+				retlist = g_list_prepend(retlist, llist->data);
+				llist = llist->next;
+			}
+		}
+		tmplist = tmplist->next;
+	}
+
+	return retlist;
+}
+
+gchar *
+langmgr_get_language_name_by_mimetype(gchar * mime)
+{
+	GList *tmplist = g_list_first(langmgr.bflang_list);
+
+	while (tmplist) {
+		if (((Tbflang *) (tmplist->data))->mimetypes) {
+			GList *llist = g_list_first(((Tbflang *) (tmplist->data))->mimetypes);
+			while (llist) {
+				if (!strcmp(llist->data, mime))
+					return ((Tbflang *) (tmplist->data))->name;
+				else
+					llist = llist->next;
+			}
+		}
+		tmplist = tmplist->next;
+	}
+
+	return NULL;
 }
 
 gboolean
