@@ -24,13 +24,13 @@
 #include "bftextview2.h"
 GList *bftextview2_scantable_rematch_highlights(Tscantable * st, const gchar * lang);
 gint16 new_context(Tscantable * st, guint expected_size, const gchar * lang, const gchar * symbols, const gchar * contexthighlight,
-				   gboolean autocomplete_case_insens, gboolean default_spellcheck);
+			gboolean autocomplete_case_insens, gboolean default_spellcheck, gboolean dump_dfa_run);
 void match_set_nextcontext(Tscantable * st, guint16 matchnum, guint16 nextcontext);
 void match_autocomplete_reference(Tscantable * st, guint16 matchnum, guint16 context);
 void match_add_autocomp_item(Tscantable * st, guint16 matchnum, const gchar * autocomplete_string,
 							 const gchar * autocomplete_append, guint8 autocomplete_backup_cursor);
 void match_set_reference(Tscantable * st, guint16 matchnum, const gchar * reference);
-void compile_existing_match(Tscantable * st, guint16 matchnum, gint16 context);
+void compile_existing_match(Tscantable * st, guint16 matchnum, gint16 context, gpointer ldb);
 void pattern_set_condition(Tscantable * st, guint16 matchnum, gchar *refname, gint relation, gint mode);
 void
 pattern_set_blockmatch(Tscantable * st, guint16 matchnum,
@@ -55,7 +55,7 @@ guint16
 add_pattern_to_scanning_table(Tscantable * st, const gchar * pattern,
 								gboolean is_regex,
 								gboolean case_insens,
-								gint16 context);
+								gint16 context, gpointer ldb);
 
 
 /*guint16 add_keyword_to_scanning_table(Tscantable * st, gchar * pattern, const gchar * lang,
@@ -68,4 +68,7 @@ void print_DFA(Tscantable * st, gint16 context, char start, char end);
 Tscantable *scantable_new(guint size_table, guint size_matches, guint size_contexts);
 Tscantable *bftextview2_scantable_new(GtkTextBuffer * buffer);
 
+void
+print_DFA_subset(Tscantable * st, gint16 context, char *chars);
+
 #endif
