@@ -130,10 +130,20 @@ static void handle_signals(void) {
 
 #ifdef MAC_INTEGRATION
 static gboolean osx_open_file_cb(GtkosxApplication *app, gchar *path, gpointer user_data) {
+	GList *tmplist;
+	Tbfwin *bfwin;
 	GFile *uri = g_file_new_for_path(path);
-	Tbfwin *bfwin = BFWIN(g_list_last(main_v->bfwinlist)->data);
+	//Find toplevel in focus
+	tmplist = g_list_first(main_v->bfwinlist);
+	while(tmplist) {
+		bfwin= BFWIN(tmplist->data);
+		if (gtk_window_is_active(GTK_WINDOW(bfwin->main_window))) {
+			break;
+		}
+		tmplist = g_list_next(tmplist);
+	}
 	g_print("osx_open_file_cb, open %s\n",path);
-	file_handle(uri, bfwin , NULL, TRUE);
+	file_handle(uri, bfwin , NULL, TRUE, FALSE);
 	g_object_unref(uri);
 	return TRUE;
 }
@@ -150,10 +160,10 @@ static gboolean osx_block_termination_cb(GtkosxApplication *app, gpointer user_d
 		tmplist = g_list_next(tmplist);
 	}
 	g_list_free(duplist);
-	return TRUE;
+	return FALSE;
 }
 
-static gboolean osx_will_terminate_cb(GtkosxApplication *app, gpointer user_data) {
+static void osx_will_terminate_cb(GtkosxApplication *app, gpointer user_data) {
 	/* Connect final shutdown routine here */
 	g_print("gtkosx terminating application");
 	flush_queue();
@@ -241,7 +251,7 @@ static gboolean startup_in_idle(gpointer data) {
 				DEBUG_MSG("startup_in_idle, we have filenames, load them\n");
 				startup->firstbfwin->focus_next_new_doc = TRUE;
 				while (tmplist) {
-					file_handle((GFile *)tmplist->data, startup->firstbfwin, NULL, TRUE);
+					file_handle((GFile *)tmplist->data, startup->firstbfwin, NULL, TRUE, FALSE);
 					g_object_unref((GFile *)tmplist->data);
 					tmplist = g_list_next(tmplist);
 				}
