@@ -2,7 +2,7 @@
  * bluefish.h - global prototypes
  *
  * Copyright (C) 1998 Olivier Sessink and Chris Mazuc
- * Copyright (C) 1999-2012 Olivier Sessink
+ * Copyright (C) 1999-2014 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,6 +116,8 @@ extern void g_none(gchar * first, ...);
 #include <sys/stat.h>
 #include <unistd.h>
 #include <gio/gio.h>
+#include <gtk/gtk.h>
+#include "compatibility.h"
 
 #define BF_FILEINFO "standard::name,standard::display-name,standard::size,standard::type,unix::mode,unix::uid,unix::gid,time::modified,time::modified-usec,etag::value,standard::fast-content-type"
 #include "bftextview2.h"
@@ -284,6 +286,7 @@ typedef struct {
 	gint editor_smart_cursor;
 	gint editor_tab_indent_sel; /* tab key will indent a selected block */
 	gint editor_auto_close_brackets;
+	gint editor_replace_unicode_quotes;
 	gint use_system_tab_font;
 	gint max_shown_filename_len; /* the maximum length for the filename shown in the tab */
 	gchar *tab_font_string;		/* notebook tabs font */
@@ -333,7 +336,6 @@ typedef struct {
 	/* not yet in use */
 	gint allow_dep;				/* allow <FONT>... */
 	gint format_by_context;		/* use <strong> instead of <b>, <emphasis instead of <i> etc. (W3C reccomendation) */
-	gint xhtml;					/* write <br /> */
 	gint smartindent;			/* add extra indent in certain situations */
 	/* key conversion */
 	gint open_in_running_bluefish;	/* open commandline documents in already running process */
@@ -347,6 +349,9 @@ typedef struct {
 	gint block_folding_mode;
 	GList *highlight_styles;
 	GList *bflang_options;		/* array with: lang_name, option_name, value */
+#ifdef WIN32
+	GList *file_association;	/* array, 0=ignored, 1=extension, 2=mimetype */
+#endif
 	gchar *autocomp_accel_string;
 	gboolean load_reference;
 	gboolean show_autocomp_reference;
@@ -367,7 +372,7 @@ typedef struct {
 	gint adv_textview_right_margin; /* advanced option for the amount of whitespace inside the textview widget */
 	gboolean wrap_on_right_margin; /* sets adv_textview_right_margin dynamically based on right_margin_pos */
 	gint adv_textview_left_margin; /* advanced option for the amount of whitespace inside the textview widget */
-	gint adv_smart_indent_mode; /* advanced option for advanced indent mode, normal=1, but if set to 2 the editor 
+	gint adv_smart_indent_mode; /* advanced option for advanced indent mode, normal=1, but if set to 2 the editor
 											will outdent if the previous character was a closing bracket */
 } Tproperties;
 
@@ -425,6 +430,7 @@ typedef struct {
 
 	gint sync_delete_deprecated;
 	gint sync_include_hidden;
+	gint sync_include_backup;
 	gint adv_open_matchname;
 	gint adv_open_recursive;
 	gint bookmarks_filename_mode;	/* 0=FULLPATH, 1=DIR FROM BASE 2=BASENAME */
@@ -445,7 +451,7 @@ typedef struct {
 	gint convertcolumn_horizontally;
 	gint display_right_margin;
 	gint show_visible_spacing;
-	/* 44 * sizeof(gint) */
+	/* 45 * sizeof(gint) */
 	/* IF YOU EDIT THIS STRUCTURE PLEASE EDIT THE CODE IN PROJECT.C THAT COPIES
 	   A Tsessionvar INTO A NEW Tsessionvar AND ADJUST THE SIZES!!!!!!!!!!!!!!!!!!!!!! */
 #ifdef HAVE_LIBENCHANT
@@ -455,6 +461,7 @@ typedef struct {
 #endif
 	/* if you add strings or lists to the session, please make sure they are free'ed
 	in free_session() in project.c */
+	gchar *snr3_filepattern;
 	gchar *ssearch_text;
 	gchar *default_mime_type;
 	gchar *template;
@@ -633,72 +640,4 @@ extern EXPORT Tmain *main_v;
 /* public functions from bluefish.c */
 void bluefish_exit_request(void);
 
-/* Avoid lots of warnings from API depreciated in GTK 3.0. -Wdeprecated-declarations */
-#if GTK_CHECK_VERSION(3,0,0)
-#define gtk_hbox_new(arg, arg2) gtk_box_new(GTK_ORIENTATION_HORIZONTAL, arg2)
-#define gtk_vbox_new(arg, arg2) gtk_box_new(GTK_ORIENTATION_VERTICAL, arg2)
-#endif
-
-/* backwards compatibility */
-#if !GTK_CHECK_VERSION(2,24,0)
-#define GDK_KEY_Enter GDK_Enter
-#define GDK_KEY_Return GDK_Return
-#define GDK_KEY_KP_Enter GDK_KP_Enter
-#define GDK_KEY_Home GDK_Home
-#define GDK_KEY_KP_Home GDK_KP_Home
-#define GDK_KEY_End GDK_End
-#define GDK_KEY_KP_End GDK_KP_End
-#define GDK_KEY_Tab GDK_Tab
-#define GDK_KEY_KP_Tab GDK_KP_Tab
-#define GDK_KEY_ISO_Left_Tab GDK_ISO_Left_Tab
-#define GDK_KEY_Up GDK_Up
-#define GDK_KEY_Down GDK_Down
-#define GDK_KEY_Page_Down GDK_Page_Down
-#define GDK_KEY_Page_Up GDK_Page_Up
-#define GDK_KEY_Right GDK_Right
-#define GDK_KEY_KP_Right GDK_KP_Right
-#define GDK_KEY_Left GDK_Left
-#define GDK_KEY_KP_Left GDK_KP_Left
-#define GDK_KEY_Escape GDK_Escape
-#define GDK_KEY_0 GDK_0
-#define GDK_KEY_1 GDK_1
-#define GDK_KEY_2 GDK_2
-#define GDK_KEY_3 GDK_3
-#define GDK_KEY_4 GDK_4
-#define GDK_KEY_5 GDK_5
-#define GDK_KEY_6 GDK_6
-#define GDK_KEY_7 GDK_7
-#define GDK_KEY_8 GDK_8
-#define GDK_KEY_9 GDK_9
-#define GDK_KEY_F1 GDK_F1
-#define GDK_KEY_F12 GDK_F12
-#define GDK_KEY_Delete GDK_Delete
-#define GDK_KEY_BackSpace GDK_BackSpace
-#define GDK_KEY_KP_Delete GDK_KP_Delete
-#define GDK_KEY_Alt_L GDK_Alt_L
-#define GDK_KEY_Alt_R GDK_Alt_R
-#define GDK_KEY_Control_L GDK_Control_L
-#define GDK_KEY_Control_R GDK_Control_R
-
-/*#define GDK_KEY_ GDK_*/
-#define GTK_COMBO_BOX_TEXT(arg) GTK_COMBO_BOX(arg)
-#define gtk_combo_box_text_get_active_text gtk_combo_box_get_active_text
-#define gtk_combo_box_text_new_with_entry gtk_combo_box_entry_new_text
-#define gtk_combo_box_text_new gtk_combo_box_new_text
-#define gtk_combo_box_text_append_text gtk_combo_box_append_text
-#define gtk_combo_box_text_prepend_text gtk_combo_box_prepend_text
-#define gtk_combo_box_text_remove gtk_combo_box_remove_text
-#endif /* GTK_CHECK_VERSION(2,24,0) */
-
-#ifdef OLD_MAC_INTEGRATION
-#define GtkosxApplication GtkOSXApplication
-#define gtkosx_application_ready gtk_osxapplication_ready
-#define GTKOSX_TYPE_APPLICATION GTK_TYPE_OSX_APPLICATION
-#define gtkosx_application_sync_menubar gtk_osxapplication_sync_menubar
-#define gtkosx_application_get_resource_path quartz_application_get_resource_path
-#define gtkosx_application_set_window_menu gtk_osxapplication_set_window_menu
-#define gtkosx_application_insert_app_menu_item gtk_osxapplication_insert_app_menu_item
-#define gtkosx_application_set_menu_bar gtk_osxapplication_set_menu_bar
-#define gtkosx_application_get_bundle_id quartz_application_get_bundle_id
-#endif
 #endif							/* __BLUEFISH_H_ */
