@@ -25,7 +25,8 @@ void outputbox_command(Tbfwin *bfwin, const gchar *formatstring);
 void filter_command(Tbfwin *bfwin, const gchar *formatstring, gint begin, gint end);
 void external_command(Tbfwin *bfwin, const gchar *formatstring);
 
-typedef void (*CustomCommandCallback) (const gchar *output, gpointer bfwin);
-void custom_command(Tbfwin *bfwin, const gchar *formatstring, CustomCommandCallback func);
+typedef void (*CustomCommandCallback) (const gchar *output, gpointer bfwin, gpointer data);
+typedef void (*StatusCodeCallback) (gint exitstatus, Tbfwin *bfwin, gpointer data);
+void custom_command(Tbfwin *bfwin, const gchar *formatstring, CustomCommandCallback func, gpointer data);
 
 #endif /* __EXTERNAL_COMMANDS_H_ */
