@@ -384,23 +384,6 @@ checkNsave_replace_async_lcb(GObject * source_object, GAsyncResult * res, gpoint
 				return;
 			}
 		}
-#if !GLIB_CHECK_VERSION(2, 18, 0)
-		else if (error->code == G_IO_ERROR_EXISTS
-				 && (g_file_has_uri_scheme(cns->uri, "sftp") || g_file_has_uri_scheme(cns->uri, "smb"))) {
-			/* there is  a bug in the GIO sftp and smb module in glib version 2.18 that returns 'file exists error'
-			   if you request an async content_replace */
-			GError *error2 = NULL;
-			g_file_replace_contents(cns->uri, cns->buffer->data, cns->buffer_size, cns->etag,
-									main_v->props.backup_file, G_FILE_CREATE_NONE, NULL, NULL, &error2);
-			if (error2) {
-				g_warning("glib < 2.18.0 workaround returns error %d: %s\n", error2->code, error2->message);
-				cns->callback_func(CHECKANDSAVE_ERROR, error2, cns->callback_data);
-				g_error_free(error2);
-			} else {
-				cns->callback_func(CHECKANDSAVE_FINISHED, NULL, cns->callback_data);
-			}
-		}
-#endif
 		else {
 			g_warning("while save to disk, received error %d: %s\n", error->code, error->message);
 			DEBUG_MSG("****************** checkNsave_replace_async_lcb() unhandled error %d: %s\n",
@@ -409,19 +392,6 @@ checkNsave_replace_async_lcb(GObject * source_object, GAsyncResult * res, gpoint
 		}
 		g_error_free(error);
 	} else {
-#if !GLIB_CHECK_VERSION(2, 18, 0)
-		/* a bug in the fuse smbnetfs mount code */
-		if (g_file_has_uri_scheme(cns->uri, "smb")) {
-			DEBUG_MSG("checkNsave_replace_async_lcb, starting glib<2.18 workaround for save on smb://\n");
-			/* check that file exists/got created */
-			if (!g_file_query_exists(cns->uri, NULL)) {
-				DEBUG_MSG("checkNsave_replace_async_lcb, smb:// workaround: save file again with backup=0\n");
-				g_file_replace_contents_async(cns->uri, cns->buffer->data, cns->buffer_size, cns->etag, FALSE	/* we already created a backup */
-											  , G_FILE_CREATE_NONE, NULL, checkNsave_replace_async_lcb, cns);
-				return;
-			}
-		}
-#endif
 		DEBUG_MSG("checkNsave_replace_async_lcb, before callback, finished with ");
 		DEBUG_URI(cns->uri, TRUE);
 		cns->callback_func(CHECKANDSAVE_FINISHED, NULL, cns->callback_data);
@@ -594,6 +564,7 @@ openfile_run(gpointer data)
 	Topenfile *of = data;
 	DEBUG_MSG("openfile_run, start async load for of=%p with uri=%p\n",of,of->uri);
 	g_file_load_contents_async(of->uri, of->cancel, openfile_async_lcb, of);
+	/*g_print("openfile_run, called g_file_load_contents_async() for uri %s, cancelable %p\n",g_file_get_uri(of->uri), of->cancel);*/
 }
 
 Topenfile *
@@ -832,10 +803,7 @@ file2doc_finished_idle_lcb(gpointer data)
 		GtkTextIter itstart, itend;
 
 		f2d->recovery_status = 2;
-		conversion_status = doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
-		if(!conversion_status) {
-			doc_set_readonly(f2d->doc, TRUE);
-		}
+		doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
 		DEBUG_MSG("file2doc_finished_idle_lcb, recovery of existing file, inserted original file\n");
 		f2d->doc->block_undo_reg = TRUE;
 		doc_unre_new_group(f2d->doc);
@@ -847,7 +815,14 @@ file2doc_finished_idle_lcb(gpointer data)
 	} else if (f2d->recovery_status == 2) {
 		conversion_status = doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
 		if(!conversion_status) {
-			doc_set_readonly(f2d->doc, TRUE);
+			g_warning("file2doc_finished_idle_lcb, file=%p failed to open, closing backend", f2d->doc);
+			bfwin_docs_not_complete(f2d->bfwin, FALSE);
+			f2d->doc->status = DOC_STATUS_ERROR;
+			f2d->doc->load = NULL;
+			doc_close_single_backend(f2d->doc, FALSE, FALSE);
+			file2doc_cleanup(data);
+			refcpointer_unref(refp);
+			return FALSE;
 		}
 		doc_unre_add(f2d->doc, f2d->buffer->data, 0, g_utf8_strlen(f2d->buffer->data, f2d->buflen), UndoInsert);
 		f2d->doc->block_undo_reg = FALSE;
@@ -866,7 +841,14 @@ file2doc_finished_idle_lcb(gpointer data)
 	} else {
 		conversion_status = doc_buffer_to_textbox(f2d->doc, f2d->buffer->data, f2d->buflen, FALSE, TRUE);
 		if(!conversion_status) {
-			doc_set_readonly(f2d->doc, TRUE);
+			g_warning("file2doc_finished_idle_lcb, file=%p failed to open, closing backend", f2d->doc);
+			bfwin_docs_not_complete(f2d->bfwin, FALSE);
+			f2d->doc->status = DOC_STATUS_ERROR;
+			f2d->doc->load = NULL;
+			doc_close_single_backend(f2d->doc, FALSE, FALSE);
+			file2doc_cleanup(data);
+			refcpointer_unref(refp);
+			return FALSE;
 		}
 		doc_reset_filetype(f2d->doc, f2d->doc->uri, f2d->buffer->data, f2d->buflen);
 		doc_set_tooltip(f2d->doc);
@@ -881,6 +863,10 @@ file2doc_finished_idle_lcb(gpointer data)
 			} else {
 				bfwin_switch_to_document_by_pointer(f2d->bfwin, f2d->doc);
 			}
+		} else if (f2d->bfwin->current_document == f2d->doc) {
+			doc_force_activate(f2d->doc);
+		} else if (f2d->doc->load_first) {
+			bfwin_switch_to_document_by_pointer(f2d->bfwin, f2d->doc);
 		}
 		{
 			gchar *utf8uri, *tmp;
@@ -978,6 +964,7 @@ fill_fileinfo_cleanup(Tfileinfo * fi)
 void
 file_asyncfileinfo_cancel(gpointer fi)
 {
+	DEBUG_MSG("file_asyncfileinfo_cancel for fi=%p and cancelable %p\n", fi, ((Tfileinfo *) fi)->cancel);
 	g_cancellable_cancel(((Tfileinfo *) fi)->cancel);
 }
 
@@ -986,6 +973,7 @@ fill_fileinfo_async_mount_lcb(GObject * source_object, GAsyncResult * res, gpoin
 {
 	Tfileinfo *fi = user_data;
 	GError *error = NULL;
+	DEBUG_MSG("fill_fileinfo_async_mount_lcb, started with fi=%p\n", fi);
 	if (g_file_mount_enclosing_volume_finish(fi->uri, res, &error)) {
 		fill_fileinfo_run(fi);
 	} else {
@@ -1026,7 +1014,13 @@ fill_fileinfo_lcb(GObject * source_object, GAsyncResult * res, gpointer user_dat
 			g_print("no file size in file info ???\n");
 		}
 #endif
-		/*doc_set_tooltip(fi->doc); */
+#ifdef PLATFORM_DARWIN
+		/* I did not noticed any issues on MacOSX.. Issue might depend on glib version (I use 2.38.0), SSD type or filesystem type
+		*  Enabled for MacOSX builds only, other platforms needs extra testing*/
+		doc_set_tooltip(fi->doc);
+#else
+		/*doc_set_tooltip(fi->doc);*/
+#endif
 	} else if (error) {
 		if (error->code == G_IO_ERROR_NOT_MOUNTED) {
 			if (gmo == NULL) {
@@ -1061,6 +1055,9 @@ static void
 fill_fileinfo_run(gpointer data)
 {
 	Tfileinfo *fi = data;
+#ifdef DEBUG
+	DEBUG_MSG("fill_fileinfo_run, calling g_file_query_info_async() for %s , fi=%p\n",g_file_get_uri(fi->uri), fi);
+#endif
 	g_file_query_info_async(fi->uri, BF_FILEINFO, G_FILE_QUERY_INFO_NONE	/* so we do follow symlinks */
 							, G_PRIORITY_LOW, fi->cancel, fill_fileinfo_lcb, fi);
 }
@@ -1141,11 +1138,11 @@ file_doc_from_uri(Tbfwin * bfwin, GFile * uri, GFile * recover_uri, GFileInfo *
 	f2d->doc->align_center = align_center;
 	f2d->doc->load_first = load_first;
 	DEBUG_MSG("file_doc_from_uri, got doc %p\n", f2d->doc);
+	f2d->of = file_openfile_uri_async(f2d->uri, bfwin, file2doc_lcb, f2d);
 	if (finfo == NULL) {
 		/* get the fileinfo also async */
 		file_doc_fill_fileinfo(f2d->doc, uri);
 	}
-	f2d->of = file_openfile_uri_async(f2d->uri, bfwin, file2doc_lcb, f2d);
 }
 
 /*************************** FIND FILES ******************************/
@@ -1175,6 +1172,7 @@ typedef struct {
 	GFile *basedir;
 	GFileEnumerator *gfe;
 	guint recursion;
+	GCancellable *cancellable;
 } Tfindfiles_dir;
 
 
@@ -1182,9 +1180,11 @@ static void
 findfiles_unref(Tfindfiles * ff)
 {
 	ff->refcount--;
-	DEBUG_MSG("openadv_unref, count=%d\n", ff->refcount);
+	DEBUG_MSG("findfiles_unref, count=%d\n", ff->refcount);
 	if (ff->refcount <= 0) {
-		ff->finished_cb(ff->data);
+		if (!ff->cancel) {
+			ff->finished_cb(ff->data);
+		}
 		ff->data=NULL;
 		ff->filematch_cb=NULL;
 		ff->finished_cb=NULL;
@@ -1213,6 +1213,7 @@ findfiles_load_directory_cleanup(Tfindfiles_dir * ffd)
 	if (ffd->gfe)
 		g_object_unref(ffd->gfe);
 	findfiles_unref(ffd->ff);
+	g_object_unref(ffd->cancellable);
 	g_slice_free(Tfindfiles_dir, ffd);
 #ifdef OAD_MEMCOUNT
 	omemcount.allocdir--;
@@ -1304,7 +1305,7 @@ enumerate_children_lcb(GObject * source_object, GAsyncResult * res, gpointer use
 		g_error_free(error);
 		findfiles_load_directory_cleanup(ffd);
 	} else {
-		g_file_enumerator_next_files_async(ffd->gfe, FFD_NUM_FILES_PER_CB, G_PRIORITY_DEFAULT + 2, NULL,
+		g_file_enumerator_next_files_async(ffd->gfe, FFD_NUM_FILES_PER_CB, G_PRIORITY_DEFAULT + 2, ffd->cancellable,
 										   enumerator_next_files_lcb, ffd);
 	}
 }
@@ -1313,7 +1314,7 @@ static void
 findfiles_rundir(gpointer data)
 {
 	Tfindfiles_dir *ffd = data;
-	g_file_enumerate_children_async(ffd->basedir, BF_FILEINFO, 0, G_PRIORITY_DEFAULT + 3, NULL,
+	g_file_enumerate_children_async(ffd->basedir, BF_FILEINFO, 0, G_PRIORITY_DEFAULT + 3, ffd->cancellable,
 									enumerate_children_lcb, ffd);
 }
 
@@ -1335,6 +1336,7 @@ findfiles_backend(Tfindfiles * ff, GFile * basedir, guint recursion)
 	ffd->recursion = recursion;
 	ffd->basedir = basedir;
 	g_object_ref(ffd->basedir);
+	ffd->cancellable = g_cancellable_new();
 
 	/* tune the queue, if there are VERY MANY files on the ofqueue, we limit the ffdqueue */
 	if (ffdqueue.max_worknum >= 8 && ofqueue.q.length > 1024)
@@ -1376,6 +1378,7 @@ findfiles(GFile *basedir, gboolean recursive, guint max_recursion, gboolean matc
 
 static void findfiles_cancel_queue_freefunc(gpointer data, gpointer user_data) {
 	Tfindfiles_dir *ffd = data;
+	g_cancellable_cancel(ffd->cancellable);
 	g_object_unref(ffd->basedir);
 	if (ffd->gfe)
 		g_object_unref(ffd->gfe);
@@ -1695,6 +1698,7 @@ typedef struct {
 	GFile *targetdir;
 	gboolean delete_deprecated;
 	gboolean include_hidden;
+	gboolean include_backup;
 	gint numworking;
 	Tasyncqueue queue_walkdir_local;
 	Tasyncqueue queue_walkdir_remote;
@@ -1972,8 +1976,10 @@ walk_local_directory_job(GIOSchedulerJob * job, GCancellable * cancellable, gpoi
 					sync_handle_error(swd->sync, swd->local_dir, "Failed to read entry in directory", error);
 			} else if (finfo) {
 				const gchar *name = g_file_info_get_name(finfo);
-				if (name && strcmp(name, "..") != 0 && strcmp(name, ".") != 0
-					&& (swd->sync->include_hidden || name[0] != '.')) {
+				gint namelen = strlen(name);
+				if (name && namelen > 0 && strcmp(name, "..") != 0 && strcmp(name, ".") != 0
+					&& (swd->sync->include_hidden || name[0] != '.')
+					&& (swd->sync->include_backup || name[namelen-1] != '~')) {
 					GFile *local, *remote;
 					GFileInfo *rfinfo;
 					local = g_file_get_child(swd->local_dir, name);
@@ -2109,7 +2115,7 @@ sync_directory_mount_lcb(GObject * source_object, GAsyncResult * res, gpointer u
 }
 
 void
-sync_directory(GFile * basedir, GFile * targetdir, gboolean delete_deprecated, gboolean include_hidden,
+sync_directory(GFile * basedir, GFile * targetdir, gboolean delete_deprecated, gboolean include_hidden, gboolean include_backup,
 			   SyncProgressCallback progress_callback, gpointer callback_data)
 {
 	GMountOperation *gmo;
@@ -2117,6 +2123,7 @@ sync_directory(GFile * basedir, GFile * targetdir, gboolean delete_deprecated, g
 	sync->refcount = 1;
 	sync->delete_deprecated = delete_deprecated;
 	sync->include_hidden = include_hidden;
+	sync->include_backup = include_backup;
 	sync->progress_callback = progress_callback;
 	sync->callback_data = callback_data;
 	queue_init(&sync->queue_walkdir_local, 3, walk_local_directory_run);
@@ -2136,7 +2143,7 @@ sync_directory(GFile * basedir, GFile * targetdir, gboolean delete_deprecated, g
 /* code to handle a file from the commandline, the filebrowser or from the message queue */
 
 void
-file_handle(GFile * uri, Tbfwin * bfwin, gchar * mimetype, gboolean explicit_open)
+file_handle(GFile * uri, Tbfwin * bfwin, gchar * mimetype, gboolean explicit_open, gboolean open_in_foreground)
 {
 	GFileInfo *finfo;
 	GError *error = NULL;
@@ -2186,7 +2193,7 @@ file_handle(GFile * uri, Tbfwin * bfwin, gchar * mimetype, gboolean explicit_ope
 			}
 		}
 	} else {
-		doc_new_from_uri(bfwin, uri, NULL, FALSE, FALSE, -1, -1, -1, TRUE, FALSE);
+		doc_new_from_uri(bfwin, uri, NULL, !open_in_foreground, FALSE, -1, -1, -1, TRUE, open_in_foreground);
 	}
 #ifdef WIN32
 	if (!mimetype)
