@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * file_dialogs.c - file dialogs
  *
- * Copyright (C) 2005-2012 Olivier Sessink
+ * Copyright (C) 2005-2014 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -368,20 +368,6 @@ file_open_url_cb(GtkAction * action, Tbfwin * bfwin)
 					 TRUE, bfwin->main_window);
 	gtk_widget_set_size_request(ou->win, 450, -1);
 	vbox = gtk_vbox_new(FALSE, 5);
-#if !GLIB_CHECK_VERSION(2, 18, 0)
-	if (glib_major_version == 2 && glib_minor_version < 18) {
-		gchar *message;
-		GtkWidget *label = gtk_label_new(NULL);
-		message =
-			g_strdup_printf(_
-							("<b>Your glib version (%d.%d.%d) works unreliable with remote files (smb, ftp, sftp, webdav etc.). Please upgrade to a glib version newer than 2.18.0 if you rely on remote file support.</b>"),
-							glib_major_version, glib_minor_version, glib_micro_version);
-		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-		gtk_label_set_markup(GTK_LABEL(label), message);
-		g_free(message);
-		gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);
-	}
-#endif
 /*	gtk_box_pack_start(GTK_BOX(vbox), bf_label_with_markup(_("<b>Open URL</b>")), FALSE, FALSE, 5);*/
 	gtk_container_add(GTK_CONTAINER(ou->win), vbox);
 	tmplist = g_list_first(bfwin->session->recent_files);
@@ -688,6 +674,7 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 {
 	gchar *tmp;
 	Trefcpointer *buffer;
+	gsize numbytes=0;
 	Tdocsavebackend *dsb;
 	GFile *dest_uri=NULL;
 	GFileInfo *dest_finfo=NULL;
@@ -826,36 +813,26 @@ doc_save_backend(Tdocument * doc, Tdocsave_mode savemode, gboolean close_doc,
 
 	session_set_savedir(doc->bfwin, dest_uri);
 
-	tmp = doc_get_buffer_in_encoding(doc);
+	tmp = doc_get_buffer_in_encoding(doc, &numbytes);
 	if (!tmp) {
 		g_free(dsb);
 		return;
 	}
-#if !GLIB_CHECK_VERSION(2, 18, 0)
-	/* check runtime glib version, check if remote file, and give warning if remote file on glib < 2.18 */
-	if (glib_major_version == 2 && glib_minor_version < 18 && !g_file_is_native(doc->uri)) {
-		gchar *message =
-			g_strdup_printf(_
-							("Your glib version (%d.%d.%d) is unreliable with remote files. Please upgrade to 2.18.0 or newer."),
-glib_major_version, glib_minor_version, glib_micro_version);
-		bfwin_statusbar_message(BFWIN(doc->bfwin), message, 20);
-		g_free(message);
-	}
-#endif
+
 	buffer = refcpointer_new(tmp);
 	doc->close_doc = close_doc;
 	doc->close_window = close_window;
 	gtk_text_view_set_editable(GTK_TEXT_VIEW(doc->view), FALSE);
 	DEBUG_MSG("doc_save_backend, calling file_checkNsave_uri_async with uri %p for %zd bytes\n", dest_uri, strlen(buffer->data));
 	doc->save =
-		file_checkNsave_uri_async(dest_uri, dest_finfo, buffer, strlen(buffer->data), savemode == docsave_normal,
+		file_checkNsave_uri_async(dest_uri, dest_finfo, buffer, numbytes, savemode == docsave_normal,
 								  main_v->props.backup_file, doc_checkNsave_lcb, dsb);
 
 	if (firstsave || savemode == docsave_saveas || savemode == docsave_move) {
 		if(doc->readonly) {
 			doc_set_readonly(doc, FALSE);
 		}
-		doc_reset_filetype(doc, doc->uri, buffer->data, strlen(buffer->data));
+		doc_reset_filetype(doc, doc->uri, buffer->data, numbytes);
 		doc_set_title(doc, NULL);
 		doc_force_activate(doc);
 	}
@@ -1284,6 +1261,7 @@ typedef struct {
 	GtkWidget *entry_remote;
 	GtkWidget *delete_deprecated;
 	GtkWidget *include_hidden;
+	GtkWidget *include_backup;
 	GtkWidget *progress;
 	GtkWidget *messagelabel;
 	gulong signal_id;
@@ -1340,19 +1318,22 @@ sync_dialog_response_lcb(GtkDialog * dialog, gint response_id, gpointer user_dat
 		if (response_id == 1) {
 			sync_directory(local, remote,
 						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->delete_deprecated)),
-						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_hidden)), sync_progress,
+						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_hidden)),
+						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_backup)), sync_progress,
 						   sd);
 		} else if (response_id == 2) {
 			sync_directory(remote, local,
 						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->delete_deprecated)),
-						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_hidden)), sync_progress,
+						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_hidden)),
+						   gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_backup)), sync_progress,
 						   sd);
 		}
 		sd->bfwin->session->sync_delete_deprecated =
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->delete_deprecated));
 		sd->bfwin->session->sync_include_hidden =
 			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_hidden));
-
+		sd->bfwin->session->sync_include_backup =
+			gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sd->include_backup));
 		g_signal_handler_block(sd->dialog, sd->signal_id);
 		g_free(sd->bfwin->session->sync_local_uri);
 		sd->bfwin->session->sync_local_uri = g_file_get_uri(local);
@@ -1382,23 +1363,7 @@ sync_dialog(Tbfwin * bfwin)
 											 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
 
 	carea = gtk_dialog_get_content_area(GTK_DIALOG(sd->dialog));
-
-#if !GLIB_CHECK_VERSION(2, 18, 0)
-	if (glib_major_version == 2 && glib_minor_version < 18) {
-		gchar *message;
-		GtkWidget *label = gtk_label_new(NULL);
-		message =
-			g_strdup_printf(_
-							("<b>Your glib version (%d.%d.%d) works unreliable with remote files (smb, ftp, sftp, webdav etc.). Please upgrade to a glib version newer than 2.18.0 if you rely on remote file support.</b>"),
-							glib_major_version, glib_minor_version, glib_micro_version);
-		gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-		gtk_label_set_markup(GTK_LABEL(label), message);
-		g_free(message);
-		gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(sd->dialog))), label, FALSE, FALSE,
-						   4);
-	}
-#endif
-	table = dialog_table_in_vbox(4, 3, 6, carea, TRUE,TRUE, 3);
+	table = dialog_table_in_vbox(5, 3, 6, carea, TRUE,TRUE, 3);
 
 	sd->entry_local = dialog_entry_in_table(NULL, table, 1, 2,0, 1);
 	dialog_mnemonic_label_in_table(_("Local directory"), sd->entry_local, table,
@@ -1419,12 +1384,18 @@ sync_dialog(Tbfwin * bfwin)
 	sd->include_hidden = dialog_check_button_in_table(_("Include hidden files"),
 						bfwin->session->sync_include_hidden, table,
 						0, 3, 3, 4);
+	sd->include_backup = dialog_check_button_in_table(_("Include backup files"),
+						bfwin->session->sync_include_backup, table,
+						0, 3, 4, 5);
 
 	sd->messagelabel = gtk_label_new(NULL);
 	gtk_box_pack_start(GTK_BOX(carea), sd->messagelabel, FALSE, FALSE, 4);
 
 	sd->progress = gtk_progress_bar_new();
 	gtk_progress_bar_set_ellipsize(GTK_PROGRESS_BAR(sd->progress), PANGO_ELLIPSIZE_MIDDLE);
+	#if GTK_CHECK_VERSION(3, 0, 0)
+	gtk_progress_bar_set_show_text (GTK_PROGRESS_BAR(sd->progress), TRUE);
+	#endif
 	gtk_box_pack_start(GTK_BOX(carea), sd->progress, FALSE, FALSE, 4);
 
 	if (bfwin->session->sync_local_uri && bfwin->session->sync_local_uri[0] != '\0') {
