@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include "bluefish.h"
 #include "file_treemodel.h"
+#include "filebrowser2.h"
 #include "bf_lib.h"
 
 static void filetreemodel_init(FileTreemodel * pkg_tree);
@@ -341,10 +342,11 @@ static void filetree_re_sort(FileTreemodel * ftm, UriRecord * precord)
 		 * Both will work, but one will give you 'jumpy'
 		 * selections after row reordering. */
 		/* neworder[(custom_list->rows[i])->pos] = i; */
-		if (neworder[i] != ((*arr)[i])->pos)
+		if (i != ((*arr)[i])->pos) {
 			reordered = TRUE;
+			DEBUG_MSG("filetree_re_sort, moved '%s' from row %d to row %d \n", (*arr)[i]->name, (*arr)[i]->pos, i);
+		}
 		neworder[i] = ((*arr)[i])->pos;
-		DEBUG_MSG("filetree_re_sort, moved '%s' from row %d to row %d \n", (*arr)[i]->name, (*arr)[i]->pos, i);
 		((*arr)[i])->pos = i;
 	}
 	if (reordered) {
@@ -457,15 +459,21 @@ static void fill_uri(UriRecord * newrecord, GFile * uri, GFileInfo * finfo)
 	/*g_print("got icon name %s from finfo %p with icon %p, content_type=%s\n",newrecord->icon_name, finfo, icon, newrecord->fast_content_type);*/
 	newrecord->weight = PANGO_WEIGHT_NORMAL;
 	newrecord->isdir = (g_file_info_get_file_type(finfo) == G_FILE_TYPE_DIRECTORY);
+	if (newrecord->isdir && strcmp(newrecord->fast_content_type, "application/octet-stream")==0) {
+		/* I've seen this happening on a sftp link and fuse mounted directories
+		such as reported in https://bugzilla.gnome.org/show_bug.cgi?id=736899 */
+		g_free(newrecord->fast_content_type);
+		newrecord->fast_content_type = g_strdup("inode/directory");
+	}
 #ifdef DEVELOPMENT
 	if (newrecord->isdir
 		&&
 		strcmp(g_file_info_get_attribute_string
 			   (finfo, G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE), "inode/directory") != 0) {
-		g_print("%s: isdir=%d but mime type =%s ???????????\n", newrecord->name, newrecord->isdir,
+		g_warning("%s: isdir=%d but mime type = %s ???????????\n", newrecord->name, newrecord->isdir,
 				g_file_info_get_attribute_string(finfo,
 												 G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE));
-		g_assert_not_reached();
+		/*g_assert_not_reached();*/
 	}
 #endif
 	/*DEBUG_MSG("fill_uri, isdir=%d for name='%s'\n",newrecord->isdir,newrecord->name); */
@@ -558,6 +566,7 @@ static gchar *get_toplevel_name(GFile * uri)
 	gmnt = g_file_find_enclosing_mount(uri, NULL, &gerror);
 	if (!gerror && gmnt) {
 		name = g_mount_get_name(gmnt);
+		DEBUG_MSG("get_toplevel_name, return %s for mount\n",name);
 		g_object_unref(gmnt);
 	} else {
 		gchar *curi = g_file_get_uri(uri);
@@ -891,6 +900,10 @@ static void enumerate_children_lcb(GObject * source_object, GAsyncResult * res,
 	GError *gerror = NULL;
 	uir->gfe = g_file_enumerate_children_finish(uir->uri, res, &gerror);
 	if (gerror) {
+		g_warning("enumerate_children_lcb, failed to list directory in filebrowser: %s (%d)\n", gerror->message, gerror->code);
+		/*check if failed uri is not basedir and reset it*/
+		fb2_check_reset_basedir(uir->uri);
+
 		if (gerror->code == 14 /* 14 = permission denied, delete any children */ ) {
 			ftm_delete_children(uir->ftm, uir->precord, FALSE);
 			uir->dir_changed = TRUE;
@@ -904,7 +917,6 @@ static void enumerate_children_lcb(GObject * source_object, GAsyncResult * res,
 			ftm_remove(uir->ftm, uir->precord, FALSE);
 			uir->dir_changed = TRUE;
 		}
-		g_warning("failed to list directory in filebrowser: %s (%d)\n", gerror->message, gerror->code);
 		g_error_free(gerror);
 		uri_in_refresh_cleanup(uir->ftm, uir);
 		return;
@@ -1011,13 +1023,16 @@ void filetreemodel_refresh_iter_async(FileTreemodel * ftm, GtkTreeIter * iter)
 		return;
 	}
 	record = iter->user_data;
-
+#ifdef DEBUG
+	DEBUG_MSG("filetreemodel_refresh_iter_async, called with iter %p which has uri with path %s\n",iter,g_file_get_uri(record->uri));
+#endif
 	refresh_dir_async(ftm, record, record->uri);
 }
 
 /*****************************************************************************
  *
  *  filetreemodel_refresh_uri_async: starts a refresh of the directory pointed to by uri.
+ *  if the uri does not yet exist in the treemodel it will abort
  *
  *****************************************************************************/
 
@@ -1077,7 +1092,9 @@ UriRecord *filetreemodel_build_dir(FileTreemodel * ftm, GFile * uri, GtkTreeIter
 		g_warning("filetreemodel_build_dir, uri == NULL, return NULL and don't set iter %p\n",iter);
 		return NULL;
 	}
-
+#ifdef DEBUG
+	DEBUG_MSG("filetreemodel_build_dir, build dir for %s\n", g_file_get_uri(uri));
+#endif
 	/* first find if any directory part of this uri exists already in the treestore */
 	g_object_ref(tmp);
 
@@ -1154,6 +1171,9 @@ void filetreemodel_add_file(FileTreemodel * ftm, GFile * uri, const gchar *conte
 
 	if (!uri)
 		return;
+#ifdef DEBUG
+	DEBUG_MSG("filetreemodel_add_file, called for uri %s\n", g_file_get_uri(uri));
+#endif
 
 	puri = g_file_get_parent(uri);
 	precord = g_hash_table_lookup(ftm->alluri, puri);
@@ -1715,7 +1735,7 @@ static void filetreemodel_finalize(GObject * object)
 	g_hash_table_destroy(ftm->alluri);
 
 	tmplist = g_list_first(ftm->uri_in_refresh);
-	DEBUG_MSG("fb2config_cleanup, stopping all async directory reads\n");
+	DEBUG_MSG("filetreemodel_finalize, stopping all async directory reads\n");
 	while (tmplist) {
 		Turi_in_refresh *uir = tmplist->data;
 		g_cancellable_cancel(uir->cancel);
