@@ -2,7 +2,7 @@
  * about.c - the About dialog
  *
  * Copyright (C) 2004 Eugene Morenko(More) more@irpin.com
- * Copyright (C) 2008-2011 Olivier Sessink
+ * Copyright (C) 2008-2014 Olivier Sessink
  * Copyright (C) 2011 James Hayward
  *
  * This program is free software; you can redistribute it and/or modify
@@ -48,10 +48,11 @@ bluefish_url_show(const gchar * url)
 		g_warning("Failed trying to launch URL in about dialog");
 	}
 #else
-#ifdef MAC_INTEGRATION
+#ifdef PLATFORM_DARWIN
 	GString *string;
-	string = g_string_new("open ");
+	string = g_string_new("open \"");
 	string = g_string_append(string, url);
+	string = g_string_append(string, "\"");
 	g_print("bluefish_url_show, launching uri=%s\n", string->str);
 	system(string->str);
 	g_string_free(string, TRUE);
@@ -135,18 +136,19 @@ about_report_bug(GtkAction * action, gpointer user_data)
 	string = g_string_new("http://bugzilla.gnome.org/enter_bug.cgi?product=bluefish");
 #ifdef WIN32
 	string = g_string_append(string, ";op_sys=Windows");
+#elif PLATFORM_DARWIN
+	string = g_string_append(string, ";op_sys=Mac%20OS");
 #endif	/* WIN32 */
 	string = g_string_append(string, ";version=");
 #ifdef SVN_REVISION
-	string = g_string_append(string, "development (SVN TRUNK)");
+	string = g_string_append_uri_escaped(string, "development (SVN TRUNK)",NULL,FALSE);
 #else	/* SVN_REVISION */
 	string = g_string_append(string, PACKAGE_VERSION);
 #endif	/* SVN_REVISION */
 	string = g_string_append(string, ";comment=");
-
 	options = g_strconcat(
 #ifdef SVN_REVISION
-						  "SVN revision ", SVN_REVISION, "\n\n"
+						  "SVN revision ", SVN_REVISION, "\n",
 #endif	/* SVN_REVISION */
 						  "Bluefish was configured with: ", CONFIGURE_OPTIONS, "\n", NULL);
 
@@ -195,9 +197,11 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 
 	const gchar *authors[] = {
 		"Olivier Sessink <olivier@bluefish.openoffice.nl> (Project leader)",
+		"Andrius <andriusr@yahoo.com>",
 		"Jim Hayward <jimhayward@linuxexperience.net>",
 		"Daniel Leidert <daniel.leidert@wgdd.de>",
 		"Shawn Novak <kernel86@gmail.com>",
+		"Frédéric Falsetti <falsetti@clansco.org>",
 		_("\nDevelopers of previous releases:"),
 		"Alastair Porter <alastair@porter.net.nz>",
 		"Antti-Juhani Kaijanaho",
@@ -221,8 +225,9 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 		"Mandrake: Todd Lyons <todd@mrball.net>",
 		"Redhat: Matthias Haase <matthias_haase@bennewitz.com>",
 		"Windows: Shawn Novak <kernel86@gmail.com>, Daniel Leidert <daniel.leidert@wgdd.de>",
-		_("\nIf you know of anyone missing from this list,\nplease let us know via <bluefish@bluefish.openoffice.nl>."),
-		_("\nThanks to all who helped making this software available."),
+		_("\nIf you know of anyone missing from this list,\nplease let us know at:"),
+		_("bluefish@bluefish.openoffice.nl <bluefish@bluefish.openoffice.nl>"),
+		_("\nThanks to all who helped make this software available.\n"),
 		NULL
 	};
 
@@ -233,11 +238,11 @@ about_dialog_create(GtkAction * action, gpointer user_data)
 		"Alastair Porter <alastair@porter.net.nz>",
 		"Daniel Blair <joecamel@realcoders.org>",
 		"Olivier Sessink <olivier@bluefish.openoffice.nl>",
-		"Denny Reeh",
+		"Denny Reeh\n",
 		NULL
 	};
 
-	const gchar *copyright = "Copyright \xc2\xa9 1998-2013 Olivier Sessink and others.\n";
+	const gchar *copyright = "Copyright \xc2\xa9 1998-2014 Olivier Sessink and others.\n";
 
 	/* wrap the license here,
 	 * the "wrap-license" property is only available with GTK >= 2.8
