@@ -337,8 +337,7 @@ static void
 htmlbar_insert_align_right_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<DIV STYLE=\"text-align: right\">") : cap("<DIV ALIGN=\"RIGHT\">"),
+						   get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("<DIV STYLE=\"text-align: right\">") : cap("<DIV ALIGN=\"RIGHT\">"),
 						   cap("</DIV>"));
 }
 
@@ -371,8 +370,7 @@ static void
 htmlbar_insert_center_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<DIV STYLE=\"text-align: center\">") : cap("<DIV ALIGN=\"CENTER\">"),
+						   get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("<DIV STYLE=\"text-align: center\">") : cap("<DIV ALIGN=\"CENTER\">"),
 						   cap("</DIV>"));
 }
 
@@ -443,18 +441,16 @@ static void
 htmlbar_insert_font_minus_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<SPAN STYLE=\"font-size: smaller;\">") : cap("<FONT SIZE=\"-1\">")
-						   , main_v->props.xhtml == 1 ? cap("</SPAN>") : cap("</FONT>"));
+						   get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("<SPAN STYLE=\"font-size: smaller;\">") : cap("<FONT SIZE=\"-1\">")
+						   , get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("</SPAN>") : cap("</FONT>"));
 }
 
 static void
 htmlbar_insert_font_plus_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<SPAN STYLE=\"font-size: larger;\">") : cap("<FONT SIZE=\"+1\">")
-						   , main_v->props.xhtml == 1 ? cap("</SPAN>") : cap("</FONT>"));
+						   get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("<SPAN STYLE=\"font-size: larger;\">") : cap("<FONT SIZE=\"+1\">")
+						   , get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("</SPAN>") : cap("</FONT>"));
 }
 
 static void
@@ -777,9 +773,8 @@ static void
 htmlbar_insert_underline_tag(GtkAction * action, gpointer user_data)
 {
 	doc_insert_two_strings(BFWIN(user_data)->current_document,
-						   main_v->props.xhtml ==
-						   1 ? cap("<SPAN STYLE=\"text-decoration: underline;\">") : cap("<U>")
-						   , main_v->props.xhtml == 1 ? cap("</SPAN>") : cap("</U>"));
+						   get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("<SPAN STYLE=\"text-decoration: underline;\">") : cap("<U>")
+						   , get_curlang_option_value(BFWIN(user_data), lang_is_XHTML) ? cap("</SPAN>") : cap("</U>"));
 }
 
 static void
@@ -809,8 +804,10 @@ htmlbar_insert_soft_hyphen(GtkAction * action, gpointer user_data)
 void
 htmlbar_toolbar_show(Thtmlbarwin * hbw, Thtmlbarsession *hbs, gboolean show)
 {
-	if (htmlbar_v.in_sidepanel)
+	if (htmlbar_v.in_sidepanel) {
+		DEBUG_MSG("htmlbar_toolbar_show, in_sidepanel=%d, nothing to do (handlebox=%p), return\n",htmlbar_v.in_sidepanel,hbw->handlebox);
 		return;
+	}
 	DEBUG_MSG("htmlbar_toolbar_show, show=%d, handlebox=%p\n",show,hbw->handlebox);
 	if (show) {
 		if (hbw->handlebox) {
@@ -1447,10 +1444,8 @@ static GtkWidget *new_html_subtoolbar(Thtmlbarwin * hbw, GtkWidget *html_noteboo
 	gtk_toolbar_set_icon_size (GTK_TOOLBAR(toolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
 #endif
 #endif
-	if (htmlbar_v.in_sidepanel) {
-		DEBUG_MSG("new_html_subtoolbar, setup vertical orientation\n");
-		gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar), GTK_ORIENTATION_VERTICAL);
-	}
+	DEBUG_MSG("new_html_subtoolbar, setup orientation need_vertical=%d,need_horizontal=%d\n",htmlbar_v.in_sidepanel,!htmlbar_v.in_sidepanel);
+	gtk_orientable_set_orientation(GTK_ORIENTABLE(toolbar), htmlbar_v.in_sidepanel?GTK_ORIENTATION_VERTICAL:GTK_ORIENTATION_HORIZONTAL);
 	label = gtk_label_new(labeltext);
 	DEBUG_MSG("new_html_subtoolbar, append toolbar to html_notebook\n");
 	gtk_notebook_append_page(GTK_NOTEBOOK(html_notebook), toolbar, label);
@@ -1469,6 +1464,7 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 
 	GtkWidget *html_notebook;
 	GtkWidget *toolbar;
+	GtkWidget *menuitem;
 
 	html_notebook = gtk_notebook_new();
 	DEBUG_MSG("htmlbar_toolbar_create, created notebook %p\n",html_notebook);
@@ -1480,9 +1476,18 @@ htmlbar_toolbar_create(Thtmlbarwin * hbw, Thtmlbarsession *hbs)
 	htmlbar_load_quickbar(hbw, hbw->quickbar_toolbar);
 	DEBUG_MSG("quickbar done\n");
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar"), _("Standard"));
+
+	if (main_v->props.allow_dep) {
+		gtk_widget_destroy(gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar/FormatStrong"));
+		gtk_widget_destroy(gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar/FormatEmphasis"));
+	} else {
+		gtk_widget_destroy(gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar/FormatBold"));
+		gtk_widget_destroy(gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLStandardToolbar/FormatItalic"));
+	}
+	
 	DEBUG_MSG("standard created\n");
 	setup_items_for_quickbar(hbw, toolbar);
-	DEBUG_MSG("standard setup done\n");
+	DEBUG_MSG("quickbar setup done\n");
 
 	toolbar = new_html_subtoolbar(hbw, html_notebook, gtk_ui_manager_get_widget(bfwin->uimanager, "/HTMLHTML5Toolbar"), _("HTML 5"));
 	setup_items_for_quickbar(hbw, toolbar);
