@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * rcfile.c - loading and parsing of the configfiles
  *
- * Copyright (C) 2000-2013 Olivier Sessink
+ * Copyright (C) 2000-2014 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -416,6 +416,7 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.editor_smart_cursor, "editor_smart_cursor:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_tab_indent_sel, "editor_tab_indent_sel:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.editor_auto_close_brackets, "editor_auto_close_brackets:", 2, TRUE);
+	init_prop_integer(&config_rc, &main_v->props.editor_replace_unicode_quotes, "editor_replace_unicode_quotes:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.use_system_tab_font, "use_system_tab_font:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.max_shown_filename_len, "max_shown_filename_len:", 0, TRUE);
 	init_prop_string(&config_rc, &main_v->props.tab_font_string, "tab_font_string:", "");
@@ -478,7 +479,6 @@ props_init_main(GHashTable * config_rc)
 	/* not yet in use */
 	init_prop_integer(&config_rc, &main_v->props.allow_dep, "allow_the_use_of_font:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.format_by_context, "format_by_context:", 1, TRUE);
-	init_prop_integer(&config_rc, &main_v->props.xhtml, "use_xhtml:", 1, TRUE);
 /*	init_prop_integer(&config_rc, &main_v->props.insert_close_tag, "insert_close_tag:", 1, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.close_tag_newline, "close_tag_newline:", 1, TRUE);*/
 	init_prop_integer(&config_rc, &main_v->props.smartindent, "smartindent:", 1, TRUE);
@@ -527,6 +527,9 @@ props_init_main(GHashTable * config_rc)
 	init_prop_integer(&config_rc, &main_v->props.wrap_on_right_margin, "wrap_on_right_margin:", 0, TRUE);
 	init_prop_integer(&config_rc, &main_v->props.adv_smart_indent_mode, "adv_smart_indent_mode:", 1, TRUE);
 	/*init_prop_arraylist(&config_rc, &main_v->props.templates, "templates:", 2, TRUE);*/
+#ifdef WIN32
+	init_prop_arraylist (&config_rc, &main_v->props.file_association, "file_association:", 3, TRUE);
+#endif
 	return config_rc;
 }
 
@@ -841,6 +844,12 @@ GList *update_commands(GList *current, gboolean overwrite)
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Firefox"), "open -a Firefox '%p'", "1", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Google Chrome"), "open -a \"Google Chrome\" '%p'", "0", NULL));
+	defaults =
+		g_list_prepend(defaults,
+					  array_from_arglist(_("Chromium"), "open -a Chromium '%p'", "0", NULL));
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Opera"), "open -a Opera '%p'", "0", NULL));
@@ -858,7 +867,7 @@ GList *update_commands(GList *current, gboolean overwrite)
 										 NULL));
 	defaults =
 		g_list_prepend(defaults,
-					  array_from_arglist(_("Chromium"), "chromium-browser '%p'&", "0", NULL));
+					  array_from_arglist(_("Chromium"), "chromium '%p' || chromium-browser '%p' &", "0", NULL));
 	defaults =
 		g_list_prepend(defaults,
 					  array_from_arglist(_("Konqueror"), "konqueror '%p'&", "0", NULL));
@@ -939,6 +948,115 @@ rcfile_parse_main(void)
 		/* TODO: list the templates in the directory */
 	}
 #endif
+
+#ifdef WIN32
+	if (main_v->props.file_association == NULL) {
+/*
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("1", ".bfproject", "application/x-bluefish-project", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("1", ".bflang2", "application/x-bluefish-language2", NULL));
+*/
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".ada", "text/x-ada", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".asp", "text/x-asp", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".sh", "text/x-shellscript", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".c", "text/x-csrc", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".h", "text/x-chdr", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".cpp", "text/x-c++src", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".cxx", "text/x-c++src", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".cc", "text/x-c++src", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".hpp", "text/x-c++hdr", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".css", "text/css", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".d", "text/x-dsrc", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".diff", "text/x-diff", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".patch", "text/x-patch", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".po", "text/x-gettext-translation", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".java", "text/x-java", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".js", "application/javascript", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".jsp", "application/x-jsp", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".mw", "text/x-mediawiki", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".nsi", "text/x-nsi", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".nsh", "text/x-nsh", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".pl", "application/x-perl", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".php", "application/x-php", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".php3", "application/x-php", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".txt", "text/plain", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".py", "text/x-python", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".rb", "text/x-ruby", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".tpl", "application/x-smarty", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".vb", "application/x-vbscript", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".xhtml", "application/xhtml+xml", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".xml", "text/xml", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						  array_from_arglist("0", ".xsl", "application/xslt+xml", NULL));
+		main_v->props.file_association =
+			g_list_append(main_v->props.file_association,
+						   array_from_arglist("0", ".xslt", "application/xslt+xml", NULL));
+	}
+#endif
 }
 
 gint
@@ -1085,6 +1203,7 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_integer(&configlist, &session->ssearch_dotmatchall, "ssearch_dotmatchall:", 0, FALSE);
 	init_prop_integer(&configlist, &session->ssearch_unescape, "ssearch_unescape:", 0, FALSE);
 	init_prop_integer(&configlist, &session->ssearch_casesens, "ssearch_casesens:", 0, FALSE);
+	init_prop_string_with_escape(&configlist, &session->snr3_filepattern, "snr_filepattern:", NULL);
 	init_prop_string_with_escape(&configlist, &session->ssearch_text, "ssearch_text:", NULL);
 
 	init_prop_integer(&configlist, &session->display_right_margin, "display_right_margin:", 0, FALSE);
@@ -1119,6 +1238,9 @@ return_session_configlist(GHashTable * configlist, Tsessionvars * session)
 	init_prop_string_with_escape(&configlist, &session->sync_local_uri, "sync_local_uri:", NULL);
 	init_prop_string_with_escape(&configlist, &session->sync_remote_uri, "sync_remote_uri:", NULL);
 	init_prop_string_with_escape(&configlist, &session->encoding, "encoding:", NULL);
+	init_prop_integer(&configlist, &session->sync_delete_deprecated, "sync_delete_deprecated:", 0, FALSE);
+	init_prop_integer(&configlist, &session->sync_include_hidden, "sync_include_hidden:", 0, FALSE);
+	init_prop_integer(&configlist, &session->sync_include_backup, "sync_include_backup:", 0, FALSE);
 	init_prop_integer(&configlist, &session->adv_open_matchname, "adv_open_matchname:", 1, FALSE);
 	init_prop_integer(&configlist, &session->adv_open_recursive, "adv_open_recursive:", 0, FALSE);
 	init_prop_string(&configlist, &session->last_filefilter, "last_filefilter:", "");
