@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snr3.c - search and replace
  *
- * Copyright (C) 2011,2012 Olivier Sessink
+ * Copyright (C) 2011,2012,2013,2014 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -888,11 +888,11 @@ threaded_all_ready(void *data) {
 	s3run->curdoc = NULL;
 	if (s3run->dialog) {
 		gchar *tmp;
+		gint count = g_queue_get_length(&s3run->results);
+		if (count == 0 && s3run->files_resultcount > 0) count =s3run->files_resultcount;
 		if (s3run->replaceall) {
-			gint count = g_queue_get_length(&s3run->results);
 			tmp = g_strdup_printf(ngettext("<i>Replaced %d entry</i>", "<i>Replaced %d entries</i>", count), count);
 		} else {
-			gint count = g_queue_get_length(&s3run->results);
 			tmp = g_strdup_printf(ngettext("<i>Found %d entry</i>", "<i>Found %d entries</i>", count), count);
 		}
 		gtk_label_set_markup(GTK_LABEL(((TSNRWin *)s3run->dialog)->searchfeedback),tmp);
@@ -1259,6 +1259,7 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		s3run->escape_chars = escapechars;
 		retval |= 1;
 	}
+	DEBUG_MSG("snr3run_init_from_gui, comparing query '%s' and '%s' for changes\n",s3run->query, query);
 	if (g_strcmp0(s3run->query, query)!=0) {
 		snr3_cancel_run(s3run);
 		g_free(s3run->query);
@@ -1316,6 +1317,8 @@ snr3run_init_from_gui(TSNRWin *snrwin, Tsnr3run *s3run)
 		snrwin->bfwin->session->snr3_escape_chars = escapechars;
 		snrwin->bfwin->session->snr3_dotmatchall = dotmatchall;
 		snrwin->bfwin->session->snr3_recursion_level = recursion_level;
+		g_free(snrwin->bfwin->session->snr3_filepattern);
+		snrwin->bfwin->session->snr3_filepattern = g_strdup(s3run->filepattern);
 	}
 
 	DEBUG_MSG("snr3run_init_from_gui, return %d\n", retval);
@@ -1621,7 +1624,7 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	currentrow++;
 	/* add a basedir and file pattern widget here */
 
-	snrwin->filepattern = combobox_with_popdown(bfwin->session->filegloblist ? g_list_last(bfwin->session->filegloblist)->data : NULL, bfwin->session->filegloblist, TRUE);
+	snrwin->filepattern = combobox_with_popdown(bfwin->session->snr3_filepattern, bfwin->session->filegloblist, TRUE);
 	snrwin->filepatternL = dialog_mnemonic_label_in_table(_("Filename pattern: "), snrwin->filepattern, table, 0, 1, currentrow, currentrow+1);
 	gtk_table_attach(GTK_TABLE(table), snrwin->filepattern, 1, 2, currentrow, currentrow+1,GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0);
 
@@ -1722,6 +1725,8 @@ snr3_advanced_dialog_backend(Tbfwin * bfwin, const gchar *findtext, Tsnr3scope s
 	g_signal_connect(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(snrwin->search))), "activate",G_CALLBACK(snr_combo_activate_lcb), snrwin);
 	g_signal_connect(gtk_bin_get_child(GTK_BIN(snrwin->replace)), "focus-out-event", G_CALLBACK(snrwin_focus_out_event_cb), snrwin);
 	g_signal_connect(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(snrwin->replace))), "activate",G_CALLBACK(snr_combo_activate_lcb), snrwin);
+	g_signal_connect(snrwin->search, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
+	g_signal_connect(snrwin->replace, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
 	g_signal_connect(snrwin->searchType, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
 	g_signal_connect(snrwin->replaceType, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
 	g_signal_connect(snrwin->scope, "changed", G_CALLBACK(snr_combobox_changed), snrwin);
